@@ -75,7 +75,7 @@ char *knh_methodop_tochar(knh_methodn_t mn)
 {
 	switch(mn) {
 	case METHODN_opNot: return "!";
-	case METHODN_opHasstanceof: return "instanceof";
+	case METHODN_opInstanceof: return "instanceof";
 	case METHODN_opAs:  return "as";
 
 	case METHODN_opHas:  return "in?";
