@@ -35,39 +35,20 @@
 extern "C" {
 #endif
 
-///* ------------------------------------------------------------------------ */
-//
-//static
-//knh_cfield_t* knh_cfield_findNULL(Ctx *ctx,knh_cfield_t *cf, size_t max, knh_fieldn_t fn)
-//{
-//	knh_index_t idx;
-//	for(idx = 0; idx < max; idx++) {
-//		if(cf[idx].fn == fn) {
-//			return cf + idx;
-//		}
-//		if(cf[idx].fn == FIELDN_NONAME) {
-//			return NULL;
-//		}
-//	}
-//	return NULL;
-//}
-//
-///* ------------------------------------------------------------------------ */
-//
-//static
-//knh_index_t knh_cfield_index(Ctx *ctx,knh_cfield_t *cf, size_t max, knh_fieldn_t fn)
-//{
-//	knh_index_t idx;
-//	for(idx = 0; idx < max; idx++) {
-//		if(cf[idx].fn == fn) {
-//			return idx;
-//		}
-//		if(cf[idx].fn == FIELDN_NONAME) {
-//			return -1;
-//		}
-//	}
-//	return -1;
-//}
+
+/* ------------------------------------------------------------------------ */
+
+knh_type_t
+knh_Compiler_typeinfer(Ctx *ctx, Compiler *cpr, knh_type_t type, char *varname)
+{
+	if(type == CLASS_Nue) type = CLASS_Any;
+
+	if(type == CLASS_Any || type == NNTYPE_Any) {
+		knh_Compiler_perror(ctx, cpr, KMSG_WTYPEINF, varname);
+	}
+
+	return type;
+}
 
 /* ------------------------------------------------------------------------ */
 
@@ -166,10 +147,6 @@ knh_cfield_t *knh_Compiler_cfieldOfVariable(Compiler *cpr, knh_index_t idx)
 	return &(DP(cpr)->vars[idx]);
 }
 
-////#define _knh_Compiler_cfieldOfGlobalVariable(mc, idx)       knh_Class_cfield(knh_Object_cid(mc->nsproto), idx)
-////#define _knh_Compiler_cfieldOfFieldVariable(mc, idx)        knh_Class_cfield(mc->this_cid, idx)
-////#define _knh_Compiler_cfieldOfLocalVariable(mc, idx)        knh_Compiler_cfieldOfVariable(b, idx)
-
 /* ======================================================================== */
 /* [name] */
 
@@ -409,21 +386,17 @@ void knh_StmtDECL_name(Ctx *ctx, Stmt *stmt, Compiler *cpr, NameSpace *ns, int l
 		if(FIELDN_IS_U1(fnq) && FIELDN_IS_U2(fnq)) {
 			flag |= KNH_FLAG_CFF_PROTECTED;
 		}
-		if(knh_Term_isASIS(StmtDECL_value(stmt)) || knh_Term_isCONST(StmtDECL_value(stmt))) {
-			Object *value = knh_Term_constValue(ctx, StmtDECL_value(stmt), var_cid);
-			if(IS_NULL(value) && TYPE_ISNOTNULL(type)) {
-				value = knh_tClass_defaultValue(ctx, var_cid);
-			}
-			knh_Compiler_declareScriptVariable(ctx, cpr, flag, type, fn, value);
+
+		Object *value = TYPE_ISNOTNULL(type) ? knh_tClass_defaultValue(ctx, var_cid) : KNH_NULL;
+		knh_Compiler_declareScriptVariable(ctx, cpr, flag, type, fn, value);
+		if(knh_Term_isASIS(StmtDECL_value(stmt))) {
 			knh_Stmt_done(ctx, stmt);
 		}
 		else {
-			knh_Compiler_declareScriptVariable(ctx, cpr, flag, type, fn, KNH_NULL);
 			knh_StmtDECL_toLET(ctx, stmt);
 		}
 	}
 	else if(level == 1) { /* FIELD VARIABLE */
-		Object *value;
 		if(FIELDN_IS_U1(fnq)) {
 			flag |= KNH_FLAG_CFF_PROTECTED;
 		}
@@ -437,15 +410,20 @@ void knh_StmtDECL_name(Ctx *ctx, Stmt *stmt, Compiler *cpr, NameSpace *ns, int l
 			return;
 		}
 
-		if(knh_Term_isASIS(StmtDECL_value(stmt)) || knh_Term_isCONST(StmtDECL_value(stmt))) {
-			value = knh_Term_constValue(ctx, StmtDECL_value(stmt), var_cid);
+		Object *value = TYPE_ISNOTNULL(type) ? knh_tClass_defaultValue(ctx, var_cid) : KNH_NULL;
+		if(knh_Term_isCONST(StmtDECL_value(stmt))) {
+			Object *v = DP(DP(stmt)->tokens[DECL_value])->data;
+			if(IS_NOTNULL(v)) {
+				if(knh_class_instanceof(knh_Object_cid(v), var_cid)) {
+					value = v;
+				}
+				else {
+					knh_Compiler_perror(ctx, cpr, KMSG_IGFIELDVALUE, FIELDN(fn));
+				}
+			}
 		}
-		else {
+		else if(!knh_Term_isASIS(StmtDECL_value(stmt))) {
 			knh_Compiler_perror(ctx, cpr, KMSG_IGFIELDVALUE, FIELDN(fn));
-			value = KNH_NULL;
-		}
-		if(IS_NULL(value) && TYPE_ISNOTNULL(type)) {
-			value = knh_tClass_defaultValue(ctx, var_cid);
 		}
 		knh_Compiler_declareVariable(ctx, cpr, flag, type, fn, value);
 		knh_Stmt_done(ctx, stmt);
@@ -530,76 +508,115 @@ void knh_StmtDECL_name(Ctx *ctx, Stmt *stmt, Compiler *cpr, NameSpace *ns, int l
 /* ======================================================================== */
 /* [LET] */
 
-#ifndef StmtMACRO
-#define StmtLET_lvalue(stmt)        DP(stmt)->tokens[0]
-#define StmtLET_rvalue(stmt)        DP(stmt)->terms[1]
-#endif
+#define GOTO_PERROR(n) { \
+		pe = n; goto L_PERROR; \
+	}\
+
+
+static
+void knh_StmtLET_addCONST(Ctx *ctx, Stmt *stmt, Compiler *cpr, NameSpace *ns, int level)
+{
+	int pe = KMSG_ESYNTAX;
+
+	if(level > 1) {
+		GOTO_PERROR(KMSG_NOTHERECONST);
+	}
+
+	knh_Stmt_terms_typing(ctx, stmt, LET_rvalue, cpr, ns, CLASS_Any);
+
+	if(!knh_Term_isCONST(DP(stmt)->terms[LET_rvalue])) {
+		GOTO_PERROR(KMSG_NNCONST);
+	}
+
+	knh_bytes_t cn = knh_Token_tobytes(DP(stmt)->tokens[LET_lvalue]);
+	knh_index_t dotidx = knh_bytes_index(cn, '.');
+	Object *value = knh_Term_constValue(ctx, DP(stmt)->terms[LET_rvalue], CLASS_Any);
+
+	if(level == 0) {
+		if(dotidx == -1) {
+			if(knh_NameSpace_existsLocalConst(ctx, ns, cn)) {
+				GOTO_PERROR(KMSG_DUPCONST);
+			}
+			String *s = (String*)DP(DP(stmt)->tokens[LET_lvalue])->data;
+			DEBUG3_ASSERT(IS_bString(s));
+			knh_NameSpace_addLocalConst(ctx, ns, s, value);
+		}
+		else {
+			knh_bytes_t fn = knh_bytes_first(cn, dotidx);
+			knh_class_t cid = knh_NameSpace_getClass(ctx, ns, fn);
+			if(cid == CLASS_unknown) {
+				GOTO_PERROR(KMSG_UCLASSN);
+			}
+			if(!knh_tClass_addClassConst(ctx, cid, knh_bytes_last(cn, dotidx+1), value)) {
+				GOTO_PERROR(KMSG_DUPCONST);
+			}
+		}
+	}
+	else if(level == 1) {
+		knh_class_t cid = DP(cpr)->this_cid;
+		if(dotidx != -1) {
+			GOTO_PERROR(KMSG_NOTHERECLASSCONST);
+		}
+		if(!knh_tClass_addClassConst(ctx, cid, knh_bytes_last(cn, dotidx+1), value)) {
+			GOTO_PERROR(KMSG_DUPCONST);
+		}
+	}
+	return ;
+
+	L_PERROR:
+	knh_Token_perror(ctx, DP(stmt)->tokens[LET_lvalue], pe);
+	return ;
+}
+
+/* ------------------------------------------------------------------------ */
 
 void knh_StmtLET_name(Ctx *ctx, Stmt *stmt, Compiler *cpr, NameSpace *ns, int level)
 {
 	if(SP(StmtLET_lvalue(stmt))->tt == TT_CONSTN) {
-		if(level > 1) {
-			knh_Compiler_perror(ctx, cpr, KMSG_NOTHERECONST, NULL);
-			knh_Stmt_done(ctx, stmt);
-			return ;
-		}
-
-		knh_Stmt_terms_typing(ctx, stmt, 1, cpr, ns, CLASS_Any);
-		if(!knh_Term_isCONST(StmtLET_rvalue(stmt))) {
-			knh_Compiler_perror(ctx, cpr, KMSG_NNCONST, NULL);
-			knh_Stmt_done(ctx, stmt);
-			return ;
-		}
-
-		Object *value = knh_Term_constValue(ctx, StmtLET_rvalue(stmt), CLASS_Any);
-		if(level == 0) {
-			knh_bytes_t cn = knh_Token_tobytes(StmtLET_lvalue(stmt));
-			knh_index_t idx = knh_bytes_index(cn, '.');
-			if(idx == -1) {
-				String *s = (String*)DP(StmtLET_lvalue(stmt))->data;
-				DEBUG3_ASSERT(IS_bString(s));
-				knh_NameSpace_addLocalConst(ctx, ns, s, value);
-			}
-			else {
-				knh_bytes_t fn = knh_bytes_first(cn, idx);
-				knh_class_t cid = knh_NameSpace_getClass(ctx, ns, fn);
-				if(cid == CLASS_unknown) {
-					knh_Token_perror(ctx, StmtLET_lvalue(stmt), KMSG_UCLASSN);
-					knh_Stmt_done(ctx, stmt);
-					return ;
-				}
-				if(!knh_tClass_addClassConst(ctx, cid, knh_bytes_last(cn, idx+1), value)) {
-					knh_Token_perror(ctx, StmtLET_lvalue(stmt), KMSG_DUPCONST);
-				}
-			}
-		}
-		else if(level == 1) {
-			knh_class_t cid = DP(cpr)->this_cid;
-			knh_bytes_t cn = knh_Token_tobytes(StmtLET_lvalue(stmt));
-			knh_index_t idx = knh_bytes_index(cn, '.');
-			if(idx != -1) {
-				knh_Compiler_perror(ctx, cpr, KMSG_NOTHERECLASSCONST, NULL);
-				knh_Stmt_done(ctx, stmt);
-				return ;
-			}
-			if(!knh_tClass_addClassConst(ctx, cid, knh_bytes_last(cn, idx+1), value)) {
-				knh_Token_perror(ctx, StmtLET_lvalue(stmt), KMSG_DUPCONST);
-			}
-		}
+		knh_StmtLET_addCONST(ctx, stmt, cpr, ns, level);
 		knh_Stmt_done(ctx, stmt);
 		return ;
-	} /* CONSTN */
+	}
 
-	if(level == 1) {
-		knh_Stmt_terms_typing(ctx, stmt, 1, cpr, ns, CLASS_Any);
-		if(knh_Term_isCONST(StmtLET_rvalue(stmt))) {
-			knh_flag_t  flag = KNH_FLAG_CFF_AUTONAME;
-			knh_fieldn_t fnq = knh_Token_tofnq(ctx, StmtLET_lvalue(stmt));
-			knh_fieldn_t fn = FIELDN_UNMASK(fnq);
-			Object *value = knh_Term_constValue(ctx, StmtLET_rvalue(stmt), CLASS_Any);
-			knh_type_t type = knh_Term_get_type(StmtLET_rvalue(stmt));
-			if(type == CLASS_Nue) type = CLASS_Any;
-			knh_notice_TypeInf(ctx, SP(stmt)->fileid, SP(stmt)->line, FIELDN(fn), type);
+	knh_fieldn_t fnq = knh_Token_tofnq(ctx, DP(stmt)->tokens[LET_lvalue]);
+	knh_flag_t  flag = KNH_FLAG_CFF_AUTONAME;
+	knh_fieldn_t fn = FIELDN_UNMASK(fnq);
+
+	if(level == 0) {  /* SCRIPT LEVEL */
+		if(knh_Compiler_indexOfScriptVariable(cpr, fnq) != -1) {
+			return ;
+		}
+		flag |= KNH_FLAG_CFF_GETTER | KNH_FLAG_CFF_SETTER;
+		if(FIELDN_IS_U2(fnq)) {
+			flag |= KNH_FLAG_CFF_PROTECTED;
+		}
+	}
+	else if(level > 1) {  /* LOCAL LEVEL */
+		if(FIELDN_IS_U1(fnq) || FIELDN_IS_U2(fnq)) {
+			return;
+		}
+		if(knh_Compiler_existsName(ctx, cpr, fnq)) {
+			return;
+		}
+	}
+
+	knh_Stmt_terms_typing(ctx, stmt, LET_rvalue, cpr, ns, CLASS_Any);
+	knh_type_t type = knh_Term_get_type(ctx, cpr, DP(stmt)->terms[LET_rvalue]);
+	type = knh_Compiler_typeinfer(ctx, cpr, type, FIELDN(fn));
+	Object *value = KNH_NULL;
+	if(knh_Term_isCONST(DP(stmt)->terms[LET_rvalue])) {
+		value = knh_Term_constValue(ctx, DP(stmt)->terms[LET_rvalue], CLASS_Any);
+	}
+
+	if(level == 0) { /* SCRIPT LEVEL */
+		Script *scr = knh_Compiler_getScript(ctx, cpr);
+		DP(cpr)->this_cid = knh_Object_cid(scr);
+		knh_Compiler_initDecl(ctx, cpr);
+		knh_Compiler_initThisVariable(ctx, cpr, DP(cpr)->this_cid);
+		knh_Compiler_declareScriptVariable(ctx, cpr, flag, type, fn, value);
+	}
+	else if(level == 1) {  /* CLASS FIELD LEVEL */
+		if(knh_Term_isCONST(DP(stmt)->terms[LET_rvalue])) {
 			if(FIELDN_IS_U1(fnq) || FIELDN_IS_U2(fnq)) {
 				flag |= KNH_FLAG_CFF_PROTECTED;
 			}
@@ -609,54 +626,9 @@ void knh_StmtLET_name(Ctx *ctx, Stmt *stmt, Compiler *cpr, NameSpace *ns, int le
 			knh_Compiler_perror(ctx, cpr, KMSG_NNCONST, NULL);
 			knh_Stmt_done(ctx, stmt);
 		}
-		return ;
 	}
-	else {
-		knh_flag_t  flag = KNH_FLAG_CFF_AUTONAME;
-		knh_fieldn_t fnq = knh_Token_tofnq(ctx, StmtLET_lvalue(stmt));
-		knh_fieldn_t fn = FIELDN_UNMASK(fnq);
-		knh_type_t   type = TYPE_Any;
-		Object *value = KNH_NULL;
-
-		if(level == 0) {
-			if(knh_Compiler_indexOfScriptVariable(cpr, fnq) != -1) {
-				return ;
-			}
-			flag |= KNH_FLAG_CFF_GETTER | KNH_FLAG_CFF_SETTER;
-			if(FIELDN_IS_U2(fnq)) {
-				flag |= KNH_FLAG_CFF_PROTECTED;
-			}
-			{
-				Script *scr = knh_Compiler_getScript(ctx, cpr);
-				DP(cpr)->this_cid = knh_Object_cid(scr);
-				knh_Compiler_initDecl(ctx, cpr);
-				knh_Compiler_initThisVariable(ctx, cpr, DP(cpr)->this_cid);
-			}
-		}
-		else if(level > 1) {
-			if(FIELDN_IS_U1(fnq) || FIELDN_IS_U2(fnq)) {
-				return;
-			}
-			if(knh_Compiler_existsName(ctx, cpr, fnq)) {
-				return;
-			}
-		}
-
-		knh_Stmt_terms_typing(ctx, stmt, 1, cpr, ns, CLASS_Any);
-		type = knh_Term_get_type(StmtLET_rvalue(stmt));
-		if(type == CLASS_Nue) type = CLASS_Any;
-		knh_notice_TypeInf(ctx, SP(stmt)->fileid, SP(stmt)->line, FIELDN(fn), type);
-
-		if(knh_Term_isCONST(StmtLET_rvalue(stmt))) {
-			value = knh_Term_constValue(ctx, StmtLET_rvalue(stmt), CLASS_Any);
-		}
-
-		if(level == 0) {
-			knh_Compiler_declareScriptVariable(ctx, cpr, flag, type, fn, value);
-		}
-		else {
-			knh_Compiler_declareVariable(ctx, cpr, flag, type, fn, value);
-		}
+	else if(level > 1) { /* LOCAL_LEVEL */
+		knh_Compiler_declareVariable(ctx, cpr, flag, type, fn, value);
 	}
 }
 
@@ -684,19 +656,22 @@ void KNH_ASM_MOV(Ctx *ctx, Compiler *cpr, Token* n, Term *v)
 	}
 }
 
+
 /* ------------------------------------------------------------------------ */
 
 void knh_StmtLET_cmpl(Ctx *ctx, Stmt *stmt, Compiler *cpr, NameSpace *ns, knh_type_t reqt, int level)
 {
-	//knh_Stmt_terms_typing(ctx, stmt, 0, cpr, ns, CLASS_Any);
-	knh_Stmt_terms_cmpl(ctx, stmt, 0, cpr, ns, CLASS_Any, 0);
-	KNH_ASSERT(IS_Token(StmtLET_lvalue(stmt)));
-	reqt = DP(StmtLET_lvalue(stmt))->type;
+	knh_Stmt_terms_cmpl(ctx, stmt, LET_lvalue, cpr, ns, CLASS_Any, 0);
+	KNH_ASSERT(IS_Token(DP(stmt)->tokens[LET_lvalue]));
+	reqt = DP(DP(stmt)->tokens[LET_lvalue])->type;
 	//knh_Stmt_terms_typing(ctx, stmt, 1, cpr, ns, TYPE_UNMASK_NN(reqt));
-	knh_Stmt_terms_cmpl(ctx, stmt, 1, cpr, ns, reqt, 0);
-	KNH_ASM_MOV(ctx, cpr, StmtLET_lvalue(stmt), StmtLET_rvalue(stmt));
-}
+	knh_Stmt_terms_cmpl(ctx, stmt, LET_rvalue, cpr, ns, reqt, 0);
+	KNH_ASM_MOV(ctx, cpr, DP(stmt)->tokens[LET_lvalue], DP(stmt)->terms[LET_rvalue]);
 
+	if(!knh_Stmt_isStatement(stmt)) {
+		knh_Stmt_terms_cmplpush(ctx, stmt, LET_lvalue, cpr, ns, reqt, level);
+	}
+}
 
 /* ======================================================================== */
 /* [letmulti] */
@@ -723,7 +698,6 @@ knh_fieldn_t knh_tName_getFieldnOnStack(Ctx *ctx, int level)
 		String *s = new_String(ctx, tname, NULL);
 		idx = knh_DictIdx_add__fast(ctx, DP(knh_rootSystem)->tfieldnDictIdx, s);
 	}
-	//DEBUG("level=%d, fn=%d", level, idx);
 	return (knh_fieldn_t)idx;
 }
 
@@ -765,7 +739,7 @@ void knh_StmtFOREACH_name(Ctx *ctx, Stmt *stmt, Compiler *cpr, NameSpace *ns, in
 					DEBUG3_ASSERT(knh_tClass[itrcid].bcid == CLASS_Iterator);
 					type = CLASS_TONNTYPE(knh_tClass[itrcid].p1);
 				}
-				knh_notice_TypeInf(ctx, SP(stmt)->fileid, SP(stmt)->line, knh_Token_tochar(StmtFOREACH_name(stmt)), type);
+				type = knh_Compiler_typeinfer(ctx, cpr, type, knh_Token_tochar(StmtFOREACH_name(stmt)));
 				knh_Compiler_declareVariable(ctx, cpr, 0, type, fn, knh_tClass_defaultValue(ctx, TYPE_UNMASK_NN(type)));
 			}
 			else {
@@ -807,7 +781,7 @@ void knh_StmtFOREACH_cmpl(Ctx *ctx, Stmt *stmt, Compiler *cpr, NameSpace *ns, kn
 		return;
 	}
 	int name_sfp = DP(StmtFOREACH_name(stmt))->index;
-	knh_type_t vtype = knh_Term_get_type(DP(stmt)->terms[FOREACH_name]);
+	knh_type_t vtype = knh_Term_get_type(ctx, cpr, DP(stmt)->terms[FOREACH_name]);
 	knh_class_t reqc = knh_class_Iterator(ctx, TYPE_UNMASK_NN(vtype));
 
 	knh_Stmt_terms_typing(ctx, stmt, FOREACH_iter, cpr, ns, reqc);
