@@ -64,17 +64,17 @@ knh_bool_t knh_Term_isTyped(Term *tm)
 
 /* ------------------------------------------------------------------------ */
 
-knh_type_t knh_Term_get_type(Term *tm)
+knh_type_t knh_Term_get_type(Ctx *ctx, Compiler *cpr, Term *tm)
 {
 	if(IS_Token(tm)) {
 		Token *tk = (Token*)tm;
-		DEBUG3_ASSERT(knh_Token_isTyped(tk));
+		SAFE_ASSERT(ctx, cpr, knh_Token_isTyped(tk));
 		return DP(tk)->type;
 	}
 	else {
 		Stmt *stmt = (Stmt*)tm;
 		DEBUG3_ASSERT(IS_Stmt(stmt));
-		DEBUG3_ASSERT(knh_Stmt_isTyped(stmt));
+		SAFE_ASSERT(ctx, cpr, knh_Stmt_isTyped(stmt));
 		return DP(stmt)->type;
 	}
 }
@@ -117,6 +117,17 @@ knh_type_t knh_Term_get_bcid(Term *tm)
 	return knh_tClass[cid].bcid;
 }
 
+/* ------------------------------------------------------------------------ */
+
+static
+int knh_StmtTERMs_isNULL(Stmt *stmt, size_t n)
+{
+	if(n < DP(stmt)->size) {
+		return (IS_Token(DP(stmt)->tokens[n]) && IS_NULL(DP(DP(stmt)->tokens[n])->data));
+	}
+	return 0;
+}
+
 /* ======================================================================== */
 /* [typing] */
 
@@ -246,11 +257,25 @@ Term *knh_StmtPARAMS_typing(Ctx *ctx, Stmt *stmt, Compiler *cpr, NameSpace *ns,
 /* ------------------------------------------------------------------------ */
 /* [CALL1] */
 
+static
 Term *knh_StmtCALL1_typing(Ctx *ctx, Stmt *stmt, Compiler *cpr, NameSpace *ns, knh_class_t reqc)
 {
 	DEBUG3_ASSERT(DP(stmt)->size == 1);
 	knh_Stmt_terms_typing(ctx, stmt, 0, cpr, ns, reqc);
 	return DP(stmt)->terms[0];
+//	knh_Stmt_setType(stmt, knh_Term_get_type(ctx, cpr, DP(stmt)->terms[0]));
+//	return TM(stmt);
+}
+
+/* ------------------------------------------------------------------------ */
+/* [LET] */
+
+static
+Term *knh_StmtLET_typing(Ctx *ctx, Stmt *stmt, Compiler *cpr, NameSpace *ns, knh_class_t reqc)
+{
+	knh_Stmt_terms_typing(ctx, stmt, 0, cpr, ns, reqc);
+	knh_Stmt_setType(stmt, knh_Term_get_type(ctx, cpr, DP(stmt)->terms[0]));
+	return TM(stmt);
 }
 
 /* ------------------------------------------------------------------------ */
@@ -315,7 +340,7 @@ Term *knh_StmtCALL_typing(Ctx *ctx, Stmt *stmt, Compiler *cpr, NameSpace *ns, kn
 			if(mn == METHODN_typeof) {
 				if(DP(stmt)->size == 3) {
 					knh_Stmt_terms_typing(ctx, stmt, 2, cpr, ns, CLASS_Any);
-					return TM(new_TokenCONST(ctx, FL(stmt), UP(new_Class__type(ctx, knh_Term_get_type(DP(stmt)->terms[2])))));
+					return TM(new_TokenCONST(ctx, FL(stmt), UP(new_Class__type(ctx, knh_Term_get_type(ctx, cpr, DP(stmt)->terms[2])))));
 				}
 				else {
 					knh_Compiler_perror(ctx, cpr, KMSG_EBUILTINFUNC, "typeof");
@@ -476,7 +501,7 @@ Term *knh_StmtCALL_typing(Ctx *ctx, Stmt *stmt, Compiler *cpr, NameSpace *ns, kn
 
 	DEBUG3_ASSERT(btype == TYPE_void);
 	knh_Stmt_terms_typing(ctx, stmt, 1, cpr, ns, CLASS_Any);
-	btype = knh_Term_get_type(DP(stmt)->terms[1]);
+	btype = knh_Term_get_type(ctx, cpr, DP(stmt)->terms[1]);
 	mtd_cid = TYPE_UNMASK_NN(btype);
 	//DEBUG3("CALL mtd_cid=%s", CLASSN(mtd_cid));
 	mtd = knh_Class_getMethod(ctx, mtd_cid, mn);
@@ -652,17 +677,26 @@ Term *knh_StmtOP_typing(Ctx *ctx, Stmt *stmt, Compiler *cpr, NameSpace *ns, knh_
 	break;
 
 	case METHODN_opEq: case METHODN_opNeq:
+		if(knh_StmtTERMs_isNULL(stmt, 2)) {
+			Term *temp = DP(stmt)->terms[1];
+			DP(stmt)->terms[1] = DP(stmt)->terms[2];
+			DP(stmt)->terms[2] = temp;
+			mtd_cid = CLASS_Nue;
+			typechecked = 1;
+			break;
+		}
+
 	case METHODN_opGt: case METHODN_opGte:
 	case METHODN_opLt: case METHODN_opLte:
 	{
-		if(opsize == 1) {
+		if(opsize != 2) {
 			knh_Compiler_perror(ctx, cpr, KMSG_EOP2, knh_token_tochar(SP(optk)->tt));
 			return knh_Stmt_foundTypeError(ctx, stmt, cpr);
 		}
 
 		mtd_cid = knh_Stmt_infer_opEq(ctx, stmt, cpr, ns);
 
-		if(mtd_cid == CLASS_Nue) {
+		if(mtd_cid == CLASS_Object) {
 			char bufcc[CLASSNAME_BUFSIZ];
 			knh_snprintf(bufcc, sizeof(bufcc), "%s %s %s",
 					CLASSN(knh_Term_get_cid(DP(stmt)->terms[1])),
@@ -742,15 +776,6 @@ Term *knh_StmtNEW_typing(Ctx *ctx, Stmt *stmt, Compiler *cpr, NameSpace *ns, knh
 		mtd_cid = knh_Token_toclass(ctx, DP(stmt)->tokens[1], CLASS_unknown, ns);
 	}
 
-//	if(knh_Token_isArrayType(DP(stmt)->tokens[1])) {
-//		if(mtd_cid == CLASS_unknown || mtd_cid == CLASS_Any) {
-//			mtd_cid = CLASS_Array;
-//		}
-//		else {
-//			mtd_cid = knh_pmzclass(ctx, CLASS_Array, mtd_cid, CLASS_Any);
-//		}
-//	}
-
 	if(mtd_cid == CLASS_unknown) {
 		knh_Token_perror(ctx, DP(stmt)->tokens[1], KMSG_UCLASSN);
 		knh_Token_perrata(ctx, DP(stmt)->tokens[0], "null");
@@ -761,7 +786,7 @@ Term *knh_StmtNEW_typing(Ctx *ctx, Stmt *stmt, Compiler *cpr, NameSpace *ns, knh
 		DEBUG_ASSERT_cid(reqc);
 		if(knh_tClass[reqc].bcid == CLASS_Array) {
 			mtd_cid = reqc;
-			knh_notice_TypeInf(ctx, SP(stmt)->fileid, SP(stmt)->line, "[]", mtd_cid);
+//			mtd_cid = knh_Compiler_typeinfer(ctx, cpr, mtd_cid, "[]");
 		}
 		else if(reqc == CLASS_Any && mn == METHODN_new__init && DP(stmt)->size > 2) {
 			knh_Stmt_terms_typing(ctx, stmt, 2, cpr, ns, CLASS_Any);
@@ -777,7 +802,7 @@ Term *knh_StmtNEW_typing(Ctx *ctx, Stmt *stmt, Compiler *cpr, NameSpace *ns, knh
 				}
 				if(icid != CLASS_Any) {
 					mtd_cid = knh_class_Array(ctx, icid);
-					knh_notice_TypeInf(ctx, SP(stmt)->fileid, SP(stmt)->line, "[]", mtd_cid);
+//					knh_notice_TypeInf(ctx, SP(stmt)->fileid, SP(stmt)->line, "[]", mtd_cid);
 				}
 			}
 		}
@@ -929,15 +954,22 @@ static
 Term *knh_StmtALT_typing(Ctx *ctx, Stmt *stmt, Compiler *cpr, NameSpace *ns, knh_class_t reqc)
 {
 	DEBUG3_ASSERT(DP(stmt)->size > 1);
-	int i;
-	knh_Stmt_terms_typing(ctx, stmt, 0, cpr, ns, reqc);
-//	if(reqc == CLASS_Any) {
-//		reqc = knh_Term_get_cid(DP(stmt)->terms[0]);
-//	}
-	for(i = 1; i < DP(stmt)->size; i++) {
+	size_t i;
+	for(i = 0; i < DP(stmt)->size; i++) {
 		knh_Stmt_terms_typing(ctx, stmt, i, cpr, ns, reqc);
 	}
-	knh_Stmt_setType(stmt, reqc);
+	if(reqc == CLASS_Any) {
+		knh_class_t icid = knh_Term_get_cid(DP(stmt)->terms[0]);
+		size_t i;
+		for(i = 1; i < DP(stmt)->size; i++) {
+			icid = knh_class_parent(icid, knh_Term_get_cid(DP(stmt)->terms[i]));
+		}
+		if(icid == CLASS_Object) icid = CLASS_Any;
+		knh_Stmt_setType(stmt, CLASS_TONNTYPE(icid));
+	}
+	else {
+		knh_Stmt_setType(stmt, CLASS_TONNTYPE(reqc));
+	}
 	return TM(stmt);
 }
 
@@ -973,6 +1005,8 @@ Term *knh_Stmt_typing(Ctx *ctx, Stmt *stmt, Compiler *cpr, NameSpace *ns, knh_cl
 	switch(stt) {
 	case STT_CALL1:
 		return knh_StmtCALL1_typing(ctx, stmt, cpr, ns, reqc);
+	case STT_LET:
+		return knh_StmtLET_typing(ctx, stmt, cpr, ns, reqc);
 	case STT_CALL:
 		return knh_StmtCALL_typing(ctx, stmt, cpr, ns, reqc);
 	case STT_NEW:
@@ -1015,7 +1049,6 @@ void knh_Stmt_terms_typing(Ctx *ctx, Stmt *stmt, size_t n, Compiler *cpr, NameSp
 	}
 
 	if(IS_Token(DP(stmt)->tokens[n])) {
-		/* knh_Token_const.c */
 		Token *tk = DP(stmt)->tokens[n];
 		if(SP(tk)->tt == TT_NAME) {
 			if(knh_Token_isSystemVariable(tk)) {
@@ -1041,10 +1074,6 @@ void knh_Stmt_terms_typing(Ctx *ctx, Stmt *stmt, size_t n, Compiler *cpr, NameSp
 			KNH_ASSERT(knh_Term_isCONST(tm));
 			KNH_SETv(ctx, DP(stmt)->stmts[n], tm);
 		}
-//		DBG2_(
-//		if(!knh_Stmt_isTyped(DP(stmt)->stmts[n])){
-//			DEBUG3("%s, n=%d, stt=%s", knh_stmt_tochar(SP(stmt)->stt), (int)n, knh_stmt_tochar(DP(DP(stmt)->stmts[n])->stt));
-//		})
 	}
 }
 
@@ -1234,8 +1263,12 @@ void knh_Stmt_perrorETYPE(Ctx *ctx, Stmt *stmt, size_t n, knh_type_t reqt)
 static
 void knh_StmtEXPR_cmpl(Ctx *ctx, Stmt *stmt, Compiler *cpr, NameSpace *ns, knh_type_t reqt, int level)
 {
-	KNH_ASSERT(knh_stmt_isExpr(SP(stmt)->stt));
-	knh_stmt_cmpl(SP(stmt)->stt)(ctx, stmt, cpr, ns, reqt, level);
+	if(knh_stmt_isExpr(SP(stmt)->stt)) {
+		knh_stmt_cmpl(SP(stmt)->stt)(ctx, stmt, cpr, ns, reqt, level);
+	}
+	else {
+		knh_Compiler_perror(ctx, cpr, KMSG_ESYNTAX, NULL);
+	}
 }
 
 /* ------------------------------------------------------------------------ */
@@ -1271,7 +1304,7 @@ void knh_Stmt_terms_cmpl_(Ctx *ctx, Stmt *stmt, int n, Compiler *cpr, NameSpace
 	{
 		Term *tm = DP(stmt)->terms[n];
 		knh_class_t reqc = TYPE_UNMASK_NN(reqt);  /*tcid*/
-		knh_type_t  vart = knh_Term_get_type(tm);
+		knh_type_t  vart = knh_Term_get_type(ctx, cpr, tm);
 		knh_class_t varc = TYPE_UNMASK_NN(vart);   /*scid*/
 
 		if(IS_Stmt(tm)) {
@@ -1289,29 +1322,6 @@ void knh_Stmt_terms_cmpl_(Ctx *ctx, Stmt *stmt, int n, Compiler *cpr, NameSpace
 			return;
 		}
 
-//		if(reqc == CLASS_Boolean) {
-//			Object *v = knh_Term_constValue(ctx, tm, CLASS_Any);
-//			if(IS_bInt(v)) {
-//				DEBUG3_ASSERT(IS_Token(DP(stmt)->tokens[n]));
-//				Token *tkc = DP(stmt)->tokens[n];
-//				knh_Token_perror(ctx, tkc, KMSG_WTRUEFALSE);
-//				if(((Int*)v)->value == 0) {
-//					knh_perrata(ctx, SP(tkc)->fileid, SP(tkc)->line, "0", "false");
-//					knh_Token_setCONST(ctx, DP(stmt)->tokens[n], KNH_FALSE);
-//				}
-//				else {
-//					char bufn[80];
-//					knh_snprintf(bufn, sizeof(bufn), "%d", (int)((Int*)v)->value);
-//					knh_perrata(ctx, SP(tkc)->fileid, SP(tkc)->line, bufn, "true");
-//					knh_Token_setCONST(ctx, DP(stmt)->tokens[n], KNH_TRUE);
-//				}
-//			}
-//			if(IS_Token(tm) && push) {
-//				KNH_ASM_PPUSH(ctx, cpr, tm, level);
-//			}
-//			return ;
-//		}
-
 		if(varc == CLASS_Nue) {
 			if(TYPE_ISNOTNULL(reqt)) {
 				knh_Stmt_perrorENULL(ctx, stmt, n);
@@ -1412,7 +1422,7 @@ void knh_StmtCALL_cmpl(Ctx *ctx, Stmt *stmt, Compiler *cpr, NameSpace *ns, knh_t
 
 	int dynamic_binding = 0;
 	if(IS_NOTNULL(mtd) && (!knh_Method_isFinal(mtd) && value == NULL)) {
-		DEBUG3("Dynamic binding");
+		//DEBUG3("Dynamic binding");
 		KNH_SETv(ctx, DP(mtdtk)->data, KNH_NULL);
 		dynamic_binding = 1;
 	}
@@ -1424,7 +1434,7 @@ void knh_StmtCALL_cmpl(Ctx *ctx, Stmt *stmt, Compiler *cpr, NameSpace *ns, knh_t
 		KNH_ASM_PUSH__DEF(ctx, cpr, mtd_cid);
 	}
 	else {
-		btype   = knh_Term_get_type(DP(stmt)->terms[1]);
+		btype   = knh_Term_get_type(ctx, cpr, DP(stmt)->terms[1]);
 		mtd_cid = TYPE_UNMASK_NN(btype);
 		if(IS_Method(mtd)) {
 			//DEBUG3("mtd_cid=%s, mtd->cid=%s", CLASSN(mtd_cid), CLASSN(DP(mtd)->cid));
@@ -1447,7 +1457,7 @@ void knh_StmtCALL_cmpl(Ctx *ctx, Stmt *stmt, Compiler *cpr, NameSpace *ns, knh_t
 		int i;
 		knh_class_t base_cid = TYPE_UNMASK_NN(btype);
 		if(knh_Method_isNullBase(mtd)) btype = base_cid;
-		DEBUG3("base_cid=%s", CLASSN(base_cid));
+		//DEBUG3("base_cid=%s", CLASSN(base_cid));
 		for(i = 2; i < DP(stmt)->size; i++) {
 			knh_type_t reqt = knh_pmztype_totype(ctx, knh_Method_ptype(mtd, i - 2), base_cid);
 			//DEBUG3("***base_cid=%s,reqt=%s%s", CLASSN(base_cid), TYPEQN(reqt));
@@ -1483,7 +1493,7 @@ void knh_StmtOP_cmpl(Ctx *ctx, Stmt *stmt, Compiler *cpr, NameSpace *ns, knh_typ
 	Method *mtd = DP(mtdtk)->mtd;
 	DEBUG3_ASSERT(IS_Method(mtd));
 	SP(stmt)->stt = STT_CALL; /* for error message  */
-	knh_type_t btype = knh_Term_get_type(DP(stmt)->terms[1]);
+	knh_type_t btype = knh_Term_get_type(ctx, cpr, DP(stmt)->terms[1]);
 	knh_class_t base_cid = TYPE_UNMASK_NN(btype);
 	if(knh_Method_isNullBase(mtd)) btype = base_cid;
 
