@@ -69,7 +69,7 @@ void knh_Token_setCONST(Ctx *ctx, Token *o, Any *data)
 
 /* ------------------------------------------------------------------------ */
 
-void knh_Token_toCONST(Token *o)
+Token* knh_Token_toCONST(Token *o)
 {
 	SP(o)->tt = TT_CONST;
 	DP(o)->type = knh_Object_cid(DP(o)->data);
@@ -77,6 +77,7 @@ void knh_Token_toCONST(Token *o)
 		DP(o)->type = CLASS_TONNTYPE(DP(o)->type);
 	}
 	knh_Token_setTyped(o, 1);
+	return o;
 }
 
 /* ======================================================================== */
