@@ -42,40 +42,37 @@ extern "C" {
 static
 Stmt *new_StmtDECL(Ctx *ctx, Token *tk_TYPEN, knh_tokens_t *tc, int start)
 {
-	Stmt *stmt_b = new_Stmt(ctx, 0, STT_DECL);
-	knh_tokens_t comma_tc = {NULL, 0, 0};
+	Stmt *stmt = new_Stmt(ctx, 0, STT_DECL);
 	if(start != 0) {
-		knh_StmtMETA_add_prestmt(ctx, stmt_b, tc, start);
+		knh_StmtMETA_add_prestmt(ctx, stmt, tc, start);
 	}
-	knh_tokens_stepFirstComma(ctx, tc, &comma_tc);
-	knh_Stmt_add(ctx, stmt_b, TM(tk_TYPEN));
-	knh_Stmt_add_VARN(ctx, stmt_b, &comma_tc);
-	if(comma_tc.c == comma_tc.e) {
-		knh_Stmt_add_ASIS(ctx, stmt_b);
+
+	knh_tokens_t expr_tc = knh_tokens_splitEXPR(ctx, tc, TT_COMMA);
+
+	knh_Stmt_add(ctx, stmt, TM(tk_TYPEN));
+	knh_Stmt_add_VARN(ctx, stmt, &expr_tc);
+
+	if(expr_tc.c == expr_tc.e) {   /* Type name; */
+		knh_Stmt_add_ASIS(ctx, stmt);
 		goto L_NEXT;
 	}
-	if(SP(comma_tc.ts[comma_tc.c])->tt == TT_LET) {
-		comma_tc.c += 1;
-		knh_Stmt_add(ctx, stmt_b, new_TermEXPR(ctx, &comma_tc, KNH_RVALUE));
+
+	if(SP(expr_tc.ts[expr_tc.c])->tt == TT_LET) {  /* Type name = (expr) */
+		expr_tc.c += 1;
+		knh_Stmt_add(ctx, stmt, new_TermEXPR(ctx, &expr_tc, KNH_RVALUE));
 	}
-	else {
-		knh_tokens_ignore(ctx, &comma_tc);
-		knh_Stmt_add_ASIS(ctx, stmt_b);
+	else { /* Syntax Error */
+		knh_Stmt_perror(ctx, stmt, KMSG_ESYNTAX, expr_tc.ts[expr_tc.c]);
+		return stmt;
 	}
 
 	L_NEXT:;
-	//DEBUG3("N tc->c=%d, tc->e=%d comma.c=%d comma.e=%d", tc->c, tc->e, comma_tc.c, comma_tc.e);
 	if(tc->c < tc->e) {
-		if(/* SP(tc->ts[tc->c])->tt == TT_SEMICOLON || */knh_Token_isBOL(tc->ts[tc->c])) {
-			return stmt_b;
-		}
-		else {
-			Stmt *stmt_n = new_StmtDECL(ctx, tk_TYPEN, tc, 0);
-			KNH_SETv(ctx, DP(stmt_n)->metaDictMap, DP(stmt_b)->metaDictMap);
-			knh_Stmt_tail_append(ctx, stmt_b, stmt_n);
-		}
+		Stmt *stmt_n = new_StmtDECL(ctx, tk_TYPEN, tc, 0);
+		KNH_SETv(ctx, DP(stmt_n)->metaDictMap, DP(stmt)->metaDictMap);
+		knh_StmtNULL_tail_append(ctx, stmt, stmt_n);
 	}
-	return stmt_b;
+	return stmt;
 }
 
 /* ======================================================================== */
@@ -135,10 +132,10 @@ Stmt *new_StmtLET(Ctx *ctx, knh_tokens_t *lvalue_tc, knh_tokens_t *rvalue_tc)
 	Term *lval = new_TermEXPR(ctx, lvalue_tc, KNH_LVALUE);
 	if(IS_Token(lval)) {
 		if(knh_Token_isLVALUE((Token*)lval)) {
-			Stmt *stmt_b = new_Stmt(ctx, 0, STT_LET);
-			knh_Stmt_add(ctx, stmt_b, lval);
-			knh_Stmt_add(ctx, stmt_b, new_TermEXPR(ctx, rvalue_tc, KNH_RVALUE));
-			return stmt_b;
+			Stmt *stmt = new_Stmt(ctx, 0, STT_LET);
+			knh_Stmt_add(ctx, stmt, lval);
+			knh_Stmt_add(ctx, stmt, new_TermEXPR(ctx, rvalue_tc, KNH_RVALUE));
+			return stmt;
 		}
 		else {
 			knh_Token_perror(ctx, (Token*)lval, KMSG_ELVALUE);
@@ -146,13 +143,13 @@ Stmt *new_StmtLET(Ctx *ctx, knh_tokens_t *lvalue_tc, knh_tokens_t *rvalue_tc)
 		}
 	}
 	else {
-		Stmt *stmt_b = (Stmt*)lval;
-		KNH_ASSERT(IS_Stmt(stmt_b));
-		knh_Stmt_toLVALUE(ctx, stmt_b, KMSG_ELVALUE);
-		if(SP(stmt_b)->stt != STT_ERR) {
-			knh_Stmt_add(ctx, stmt_b, new_TermEXPR(ctx, rvalue_tc, KNH_RVALUE));
+		Stmt *stmt = (Stmt*)lval;
+		KNH_ASSERT(IS_Stmt(stmt));
+		knh_Stmt_toLVALUE(ctx, stmt, KMSG_ELVALUE);
+		if(SP(stmt)->stt != STT_ERR) {
+			knh_Stmt_add(ctx, stmt, new_TermEXPR(ctx, rvalue_tc, KNH_RVALUE));
 		}
-		return stmt_b;
+		return stmt;
 	}
 }
 
@@ -171,20 +168,16 @@ Stmt *new_StmtLETOP(Ctx *ctx, knh_tokens_t *lvalue_tc, Token *tkl, knh_tokens_t
 static
 Stmt *new_StmtLETMULTI(Ctx *ctx, knh_tokens_t *lvalue_tc, knh_tokens_t *rvalue_tc)
 {
-	knh_tokens_t cma_tc;
-	knh_tokens_stepFirstComma(ctx, lvalue_tc, &cma_tc);
-	{
-		Stmt *stmt_head = new_StmtLET(ctx, &cma_tc, rvalue_tc);
-		Stmt *stmt_b = new_Stmt(ctx, 0, STT_LETMULTI);
-		while(cma_tc.c < cma_tc.e) {
-			DEBUG3_STC("cma_tc", cma_tc);
-			knh_Stmt_add_VARN(ctx, stmt_b, &cma_tc);
-			knh_tokens_ignore(ctx, &cma_tc);
-			knh_tokens_stepFirstComma(ctx, lvalue_tc, &cma_tc);
-		}
-		knh_Stmt_tail_append(ctx, stmt_head, stmt_b);
-		return stmt_head;
+	knh_tokens_t cma_tc = knh_tokens_splitEXPR(ctx, lvalue_tc, TT_COMMA);
+	Stmt *stmt_head = new_StmtLET(ctx, &cma_tc, rvalue_tc);
+	Stmt *stmt = new_Stmt(ctx, 0, STT_LETMULTI);
+	while(cma_tc.c < cma_tc.e) {
+		DEBUG3_STC("cma_tc", cma_tc);
+		knh_Stmt_add_VARN(ctx, stmt, &cma_tc);
+		cma_tc = knh_tokens_splitEXPR(ctx, lvalue_tc, TT_COMMA);
 	}
+	knh_StmtNULL_tail_append(ctx, stmt_head, stmt);
+	return stmt_head;
 }
 
 /* ------------------------------------------------------------------------ */
@@ -193,22 +186,14 @@ static
 Stmt *new_StmtLETMULTI2(Ctx *ctx, knh_tokens_t *lvalue_tc, knh_tokens_t *rvalue_tc)
 {
 	Stmt *stmt_head = NULL;
-	knh_tokens_t ltc, rtc;
-	knh_tokens_stepFirstComma(ctx, lvalue_tc, &ltc);
-	knh_tokens_stepFirstComma(ctx, rvalue_tc, &rtc);
+	knh_tokens_t ltc = knh_tokens_splitEXPR(ctx, lvalue_tc, TT_COMMA);
+	knh_tokens_t rtc = knh_tokens_splitEXPR(ctx, rvalue_tc, TT_COMMA);
 	while(ltc.c < ltc.e && rtc.c < rtc.e) {
-		Stmt *stmt_b = new_StmtLET(ctx, &ltc, &rtc);
-		if(stmt_head == NULL) {
-			stmt_head = stmt_b;
-		}
-		else {
-			knh_Stmt_tail_append(ctx, stmt_head, stmt_b);
-		}
-		knh_tokens_stepFirstComma(ctx, lvalue_tc, &ltc);
-		knh_tokens_stepFirstComma(ctx, rvalue_tc, &rtc);
+		Stmt *stmt = new_StmtLET(ctx, &ltc, &rtc);
+		stmt_head = knh_StmtNULL_tail_append(ctx, stmt_head, stmt);
+		ltc = knh_tokens_splitEXPR(ctx, lvalue_tc, TT_COMMA);
+		rtc = knh_tokens_splitEXPR(ctx, rvalue_tc, TT_COMMA);
 	}
-	knh_tokens_ignore(ctx, lvalue_tc);
-	knh_tokens_ignore(ctx, rvalue_tc);
 	KNH_ASSERT(stmt_head != NULL);
 	return stmt_head;
 }
@@ -250,11 +235,9 @@ void knh_tokens_divideLET(knh_tokens_t *tc, knh_tokens_t *ltc, knh_tokens_t *rtc
 
 Stmt* new_StmtLETEXPR(Ctx *ctx, knh_tokens_t *tc)
 {
-	Token *tkl = NULL;
-	knh_tokens_t expr_tc;
-	knh_tokens_stepFirstStmt(ctx, tc, &expr_tc, 1);
+	knh_tokens_t expr_tc = knh_tokens_splitSTMT(ctx, tc);
+	Token *tkl = knh_tokens_findLETNULL(&expr_tc);
 
-	tkl = knh_tokens_findLETNULL(&expr_tc);
 	if(tkl != NULL) {
 		knh_tokens_t lvalue_tc, rvalue_tc;
 		knh_tokens_divideLET(&expr_tc, &lvalue_tc, &rvalue_tc);
@@ -275,8 +258,12 @@ Stmt* new_StmtLETEXPR(Ctx *ctx, knh_tokens_t *tc)
 			}
 		}
 	}
+	else if(!(expr_tc.c < expr_tc.e)) {
+		return new_Stmt(ctx, 0, STT_DONE);
+	}
 	else {
-		Term *term = new_TermEXPR(ctx, &expr_tc, KNH_STT_RVALUE);
+
+		Term *term = new_TermEXPR(ctx, &expr_tc, KNH_RVALUE);
 		if(IS_Token(term)) {
 			Stmt *stmt = new_Stmt(ctx, 0, STT_CALL1);
 			knh_Stmt_add(ctx, stmt, term);
@@ -289,15 +276,27 @@ Stmt* new_StmtLETEXPR(Ctx *ctx, knh_tokens_t *tc)
 
 /* ------------------------------------------------------------------------ */
 
-void knh_Stmt_add_EXPR(Ctx *ctx, Stmt *stmt_b, knh_tokens_t *tc)
+void knh_Stmt_add_EXPR(Ctx *ctx, Stmt *stmt, knh_tokens_t *tc)
 {
-	knh_tokens_t expr_tc;
-	knh_tokens_stepFirstStmt(ctx, tc, &expr_tc, 1);
+	knh_tokens_t expr_tc = knh_tokens_splitSTMT(ctx, tc);
 	if(expr_tc.c < expr_tc.e) {
-		knh_Stmt_add(ctx, stmt_b, new_TermEXPR(ctx, &expr_tc, KNH_RVALUE));
+		knh_Stmt_add(ctx, stmt, new_TermEXPR(ctx, &expr_tc, KNH_RVALUE));
 	}
 	else {
-		knh_tokens_perror(ctx, tc, KMSG_EEMPTYEXPR);
+		knh_Stmt_perror(ctx, stmt, KMSG_WEMPTY, NULL);
+	}
+}
+
+/* ------------------------------------------------------------------------ */
+
+void knh_Stmt_add_EXPRs(Ctx *ctx, Stmt *stmt_b, knh_tokens_t *tc)
+{
+	knh_tokens_t expr_tc = knh_tokens_splitSTMT(ctx, tc);
+	while(expr_tc.c < expr_tc.e) {
+		knh_tokens_t sub_tc = knh_tokens_splitEXPR(ctx, &expr_tc, TT_COMMA);
+		if(sub_tc.c < sub_tc.e) {
+			knh_Stmt_add(ctx, stmt_b, new_TermEXPR(ctx, &sub_tc, KNH_RVALUE));
+		}
 	}
 }
 
@@ -481,9 +480,9 @@ Term *new_TermNAME(Ctx *ctx, Token *tk, int lr)
 			return TM(tk);
 		}
 		else {
-			Stmt* stmt_b = new_Stmt(ctx, 0, STT_CALL);
-			knh_Stmt_add_FIRSTNAME(ctx, stmt_b, tk, t, lr);
-			return TM(stmt_b);
+			Stmt* stmt = new_Stmt(ctx, 0, STT_CALL);
+			knh_Stmt_add_FIRSTNAME(ctx, stmt, tk, t, lr);
+			return TM(stmt);
 		}
 	}
 }
@@ -497,14 +496,14 @@ Stmt *new_StmtNAME1(Ctx *ctx, Token *tk, int lr)
 	{
 		knh_bytes_t t = knh_Token_tobytes(tk);
 		if(knh_bytes_rindex(t, '.') == -1) {
-			Stmt *stmt_b = new_Stmt(ctx, 0, STT_CALL);
-			knh_Stmt_add(ctx, stmt_b, TM(tk));
-			return stmt_b;
+			Stmt *stmt = new_Stmt(ctx, 0, STT_CALL);
+			knh_Stmt_add(ctx, stmt, TM(tk));
+			return stmt;
 		}
 		else {
-			Stmt* stmt_b = new_Stmt(ctx, 0, STT_CALL);
-			knh_Stmt_add_FIRSTNAME(ctx, stmt_b, tk, t, lr);
-			return new_Stmt__stmt(ctx, stmt_b);
+			Stmt* stmt = new_Stmt(ctx, 0, STT_CALL);
+			knh_Stmt_add_FIRSTNAME(ctx, stmt, tk, t, lr);
+			return new_Stmt__stmt(ctx, stmt);
 		}
 	}
 }
@@ -517,15 +516,15 @@ Stmt *new_StmtNAME2(Ctx *ctx, Token *tk, int lr)
 	DEBUG3_ASSERT(SP(tk)->tt == TT_NAME || SP(tk)->tt == TT_CMETHODN);
 	knh_bytes_t t = knh_Token_tobytes(tk);
 	if(knh_bytes_rindex(t, '.') == -1) {
-		Stmt *stmt_b = new_Stmt(ctx, 0, STT_CALL);
-		knh_Stmt_add(ctx, stmt_b, TM(tk));
-		knh_Stmt_add(ctx, stmt_b, TM(new_TokenTHIS(ctx, tk)));
-		return stmt_b;
+		Stmt *stmt = new_Stmt(ctx, 0, STT_CALL);
+		knh_Stmt_add(ctx, stmt, TM(tk));
+		knh_Stmt_add(ctx, stmt, TM(new_TokenTHIS(ctx, tk)));
+		return stmt;
 	}
 	else {
-		Stmt* stmt_b = new_Stmt(ctx, 0, STT_CALL);
-		knh_Stmt_add_FIRSTNAME(ctx, stmt_b, tk, t, lr);
-		return stmt_b;
+		Stmt* stmt = new_Stmt(ctx, 0, STT_CALL);
+		knh_Stmt_add_FIRSTNAME(ctx, stmt, tk, t, lr);
+		return stmt;
 	}
 }
 
@@ -546,24 +545,24 @@ Term *new_TermTUPLE(Ctx *ctx, Token *tk)
 		return new_TermEXPR(ctx, &tc, KNH_RVALUE);
 	}
 	else {
-		Stmt *stmt_b = new_Stmt(ctx, 0, STT_NEW);
+		Stmt *stmt = new_Stmt(ctx, 0, STT_NEW);
 		Token *tk_new = new_TokenMN(ctx, FL(tk), METHODN_new__init);
-		knh_Stmt_add(ctx, stmt_b, TM(tk_new));
-		knh_Stmt_add(ctx, stmt_b, TM(tk_new) /* DUMMY new_Token__T(ctx, tk, STEXT("Tuple"))*/);
-		knh_Stmt_add_EXPRs(ctx, stmt_b, &tc);
-		c = DP(stmt_b)->size - 2;
+		knh_Stmt_add(ctx, stmt, TM(tk_new));
+		knh_Stmt_add(ctx, stmt, TM(tk_new) /* DUMMY new_Token__T(ctx, tk, STEXT("Tuple"))*/);
+		knh_Stmt_add_EXPRs(ctx, stmt, &tc);
+		c = DP(stmt)->size - 2;
 		if(c == 2) {
-			KNH_SETv(ctx, DP(stmt_b)->terms[1], new_TokenCID(ctx, FL(tk), CLASS_Tuple2));
+			KNH_SETv(ctx, DP(stmt)->terms[1], new_TokenCID(ctx, FL(tk), CLASS_Tuple2));
 		}
 #ifdef CLASS_Triple
 		else if(c == 3){
-			KNH_SETv(ctx, DP(stmt_b)->terms[1], new_TokenCID(ctx, FL(tk), CLASS_Triple));
+			KNH_SETv(ctx, DP(stmt)->terms[1], new_TokenCID(ctx, FL(tk), CLASS_Triple));
 		}
 #endif
 		else {
-			KNH_SETv(ctx, DP(stmt_b)->terms[1], new_TokenCID(ctx, FL(tk), CLASS_Array));
+			KNH_SETv(ctx, DP(stmt)->terms[1], new_TokenCID(ctx, FL(tk), CLASS_Array));
 		}
-		return TM(stmt_b);
+		return TM(stmt);
 	}
 }
 
@@ -592,169 +591,241 @@ Stmt *new_StmtARRAY(Ctx *ctx, Token *tk)
 	Token *tkidx;
 	knh_Token_tc(tk, &tc);
 	if((tkidx = knh_tokens_findIDXNULL(&tc)) == NULL) {
-		Stmt *stmt_b = new_Stmt(ctx, 0, STT_NEW);
-		knh_Stmt_add(ctx, stmt_b, TM(new_TokenMN(ctx, FL(tk), METHODN_new__init)));
-		knh_Stmt_add(ctx, stmt_b, TM(new_TokenCID(ctx, FL(tk), CLASS_Array)));
-		knh_Stmt_add_EXPRs(ctx, stmt_b, &tc);
-		return stmt_b;
+		Stmt *stmt = new_Stmt(ctx, 0, STT_NEW);
+		knh_Stmt_add(ctx, stmt, TM(new_TokenMN(ctx, FL(tk), METHODN_new__init)));
+		knh_Stmt_add(ctx, stmt, TM(new_TokenCID(ctx, FL(tk), CLASS_Array)));
+		knh_Stmt_add_EXPRs(ctx, stmt, &tc);
+		return stmt;
 	}
 	else {
-		Stmt *stmt_b = new_Stmt(ctx, 0, STT_NEW);
-		knh_tokens_t first_tc;
-		knh_tokens_stepFirstEMPTYEXPR(ctx, &tc, SP(tkidx)->tt, &first_tc);
+		Stmt *stmt = new_Stmt(ctx, 0, STT_NEW);
+		knh_tokens_t first_tc = knh_tokens_splitEXPR(ctx, &tc, SP(tkidx)->tt);
 		if(SP(tkidx)->tt == TT_SUBSETE) {
-			knh_Stmt_add(ctx, stmt_b, TM(new_TokenMN(ctx, FL(tk), METHODN_new)));
+			knh_Stmt_add(ctx, stmt, TM(new_TokenMN(ctx, FL(tk), METHODN_new)));
 		}
 		else if(SP(tkidx)->tt == TT_OFFSET) {
-			knh_Stmt_add(ctx, stmt_b, TM(new_TokenMN(ctx, FL(tk), METHODN_new__offset)));
+			knh_Stmt_add(ctx, stmt, TM(new_TokenMN(ctx, FL(tk), METHODN_new__offset)));
 		}
 		else {
-			knh_Stmt_add(ctx, stmt_b, TM(new_TokenMN(ctx, FL(tk), METHODN_new__slice)));
+			knh_Stmt_add(ctx, stmt, TM(new_TokenMN(ctx, FL(tk), METHODN_new__slice)));
 		}
-		knh_Stmt_add(ctx, stmt_b, TM(new_TokenCID(ctx, FL(tk), CLASS_Range)));
+		knh_Stmt_add(ctx, stmt, TM(new_TokenCID(ctx, FL(tk), CLASS_Range)));
 		if(first_tc.c == first_tc.e) {
-			knh_Stmt_add(ctx, stmt_b, TM(new_TokenCONST(ctx, FL(tkidx), KNH_NULL)));
+			knh_Stmt_add(ctx, stmt, TM(new_TokenCONST(ctx, FL(tkidx), KNH_NULL)));
 		}
 		else {
-			knh_Stmt_add(ctx, stmt_b, new_TermEXPR(ctx, &first_tc, KNH_RVALUE));
+			knh_Stmt_add(ctx, stmt, new_TermEXPR(ctx, &first_tc, KNH_RVALUE));
 		}
 		if(tc.c == tc.e) {
-			knh_Stmt_add(ctx, stmt_b, TM(new_TokenCONST(ctx, FL(tkidx), KNH_NULL)));
+			knh_Stmt_add(ctx, stmt, TM(new_TokenCONST(ctx, FL(tkidx), KNH_NULL)));
 		}
 		else {
-			knh_Stmt_add(ctx, stmt_b, new_TermEXPR(ctx, &tc, KNH_RVALUE));
+			knh_Stmt_add(ctx, stmt, new_TermEXPR(ctx, &tc, KNH_RVALUE));
 		}
-		return stmt_b;
+		return stmt;
 	}
 }
 
 /* ======================================================================== */
 
 static
-void knh_Stmt_add_IDX(Ctx *ctx, Stmt *stmt_b, Token *tkb, int lr)
+void knh_Stmt_add_IDX(Ctx *ctx, Stmt *stmt, Token *tkb, int lr)
 {
 	KNH_ASSERT(SP(tkb)->tt == TT_BRANCET);
-	{
-		knh_tokens_t tc;
-		Token *tkidx = NULL;
+	knh_tokens_t tc;
 
-		knh_Token_tc(tkb, &tc);
-		if(tc.e == 0) {
-			knh_Stmt_add(ctx, stmt_b, TM(new_TokenMN(ctx, FL(tkb), METHODN_getSize)));
-			knh_Stmt_swap01(stmt_b);
-			return ;
+	Token *tkidx = NULL;
+	knh_Token_tc(tkb, &tc);
+	if(tc.e == 0) {
+		knh_Stmt_add(ctx, stmt, TM(new_TokenMN(ctx, FL(tkb), METHODN_setAll)));
+		knh_Stmt_swap01(stmt);
+		return ;
+	}
+	if((tkidx = knh_tokens_findIDXNULL(&tc)) == NULL) {
+		if(lr == KNH_LVALUE) {
+			knh_Stmt_add(ctx, stmt, TM(new_TokenMN(ctx, FL(tkb), METHODN_set)));
 		}
-		if((tkidx = knh_tokens_findIDXNULL(&tc)) == NULL) {
+		else{
+			knh_Stmt_add(ctx, stmt, TM(new_TokenMN(ctx, FL(tkb), METHODN_get)));
+		}
+		knh_Stmt_swap01(stmt);
+		knh_Stmt_add_EXPRs(ctx, stmt, &tc);
+#ifdef METHODN_set2D
+		if(DP(stmt)->size == 4) {
 			if(lr == KNH_LVALUE) {
-				knh_Stmt_add(ctx, stmt_b, TM(new_TokenMN(ctx, FL(tkb), METHODN_set)));
+				DP(DP(stmt)->tokens[0])->mn = METHODN_set2D;
 			}
 			else{
-				knh_Stmt_add(ctx, stmt_b, TM(new_TokenMN(ctx, FL(tkb), METHODN_get)));
+				DP(DP(stmt)->tokens[0])->mn = METHODN_get2D;
 			}
-			knh_Stmt_swap01(stmt_b);
-			knh_Stmt_add_EXPRs(ctx, stmt_b, &tc);
-//			if(DP(stmt_b)->size == 4) {
-//				if(lr == KNH_LVALUE) {
-//					DP(DP(stmt_b)->tokens[0])->mn = METHODN_set__2;
-//				}
-//				else{
-//					DP(DP(stmt_b)->tokens[0])->mn = METHODN_get__2;
-//				}
-//			}
-//			else if(DP(stmt_b)->size == 5){
-//				if(lr == KNH_LVALUE) {
-//					DP(DP(stmt_b)->tokens[0])->mn = METHODN_set__3;
-//				}
-//				else{
-//					DP(DP(stmt_b)->tokens[0])->mn = METHODN_set__3;
-//				}
-//			}
+		}
+		else if(DP(stmt)->size == 5){
+			if(lr == KNH_LVALUE) {
+				DP(DP(stmt)->tokens[0])->mn = METHODN_set3D;
+			}
+			else{
+				DP(DP(stmt)->tokens[0])->mn = METHODN_get3D;
+			}
+		}
+#endif
+	}
+	else {
+		knh_tokens_t first_tc = knh_tokens_splitEXPR(ctx, &tc, SP(tkidx)->tt);
+		if(SP(tkidx)->tt == TT_SUBSETE) {
+			knh_Stmt_add(ctx, stmt, TM(new_TokenMN(ctx, FL(tkb), METHODN_opSubsete)));
+		}
+		else if(SP(tkidx)->tt == TT_OFFSET) {
+			knh_Stmt_add(ctx, stmt, TM(new_TokenMN(ctx, FL(tkb), METHODN_opOffset)));
 		}
 		else {
-			knh_tokens_t first_tc;
-			knh_tokens_stepFirstEMPTYEXPR(ctx, &tc, SP(tkidx)->tt, &first_tc);
-			if(SP(tkidx)->tt == TT_SUBSETE) {
-				knh_Stmt_add(ctx, stmt_b, TM(new_TokenMN(ctx, FL(tkb), METHODN_opSubsete)));
-			}
-			else if(SP(tkidx)->tt == TT_OFFSET) {
-				knh_Stmt_add(ctx, stmt_b, TM(new_TokenMN(ctx, FL(tkb), METHODN_opOffset)));
-			}
-			else {
-				knh_Stmt_add(ctx, stmt_b, TM(new_TokenMN(ctx, FL(tkb), METHODN_opSubset)));
-			}
-			knh_Stmt_swap01(stmt_b);
-			if(first_tc.c == first_tc.e) {
-				knh_Stmt_add(ctx, stmt_b, TM(new_TokenCONST(ctx, FL(tkidx), KNH_NULL)));
-			}
-			else {
-				knh_Stmt_add(ctx, stmt_b, new_TermEXPR(ctx, &first_tc, KNH_RVALUE));
-			}
-			if(tc.c == tc.e) {
-				knh_Stmt_add(ctx, stmt_b, TM(new_TokenCONST(ctx, FL(tkidx), KNH_NULL)));
-			}
-			else {
-				knh_Stmt_add(ctx, stmt_b, new_TermEXPR(ctx, &tc, KNH_RVALUE));
-			}
+			knh_Stmt_add(ctx, stmt, TM(new_TokenMN(ctx, FL(tkb), METHODN_opSubset)));
+		}
+		knh_Stmt_swap01(stmt);
+		if(first_tc.c == first_tc.e) {
+			knh_Stmt_add(ctx, stmt, TM(new_TokenCONST(ctx, FL(tkidx), KNH_NULL)));
+		}
+		else {
+			knh_Stmt_add(ctx, stmt, new_TermEXPR(ctx, &first_tc, KNH_RVALUE));
+		}
+		if(tc.c == tc.e) {
+			knh_Stmt_add(ctx, stmt, TM(new_TokenCONST(ctx, FL(tkidx), KNH_NULL)));
+		}
+		else {
+			knh_Stmt_add(ctx, stmt, new_TermEXPR(ctx, &tc, KNH_RVALUE));
 		}
-		knh_Token_tokens_empty(ctx, tkb);
 	}
+	knh_Token_tokens_empty(ctx, tkb);
 }
 
 /* ======================================================================== */
-/* [HashMap] */
+/* [DictMap] */
 
-static
-void knh_tokens_stepFirstColon(knh_tokens_t *tc, knh_tokens_t *first_tc)
+void knh_tokens_toArray(Ctx *ctx, knh_tokens_t *tc, Array *a, int ignoreCOMMA)
 {
 	int i;
-	first_tc->c = tc->c;
-	first_tc->ts = tc->ts;
+	Token **ts = tc->ts;
 	for(i = tc->c; i < tc->e; i++) {
-		if(SP(tc->ts[i])->tt == TT_LABEL) {
-			first_tc->e = i+1;
-			tc->c = i + 1;
-			return;
+		if(ignoreCOMMA && SP(ts[i])->tt == TT_COMMA) continue;
+		if(SP(ts[i])->tt == TT_PARENTHESIS) {
+			knh_tokens_t intc;
+			knh_Array_add(ctx, a, UP(TS_LP));
+			knh_Token_tc(ts[i], &intc);
+			knh_tokens_toArray(ctx, &intc, a, 0);
+			knh_Array_add(ctx, a, UP(TS_RP));
 		}
-		if(SP(tc->ts[i])->tt == TT_COLON) {
-			first_tc->e = i;
-			tc->c = i + 1;
-			return ;
+		else if(SP(ts[i])->tt == TT_BRACE) {
+			knh_tokens_t intc;
+			knh_Array_add(ctx, a, UP(TS_LB));
+			knh_Token_tc(ts[i], &intc);
+			knh_tokens_toArray(ctx, &intc, a, 0);
+			knh_Array_add(ctx, a, UP(TS_RB));
+		}
+		else if(SP(ts[i])->tt == TT_BRANCET) {
+			knh_tokens_t intc;
+			knh_Array_add(ctx, a, UP(TS_LS));
+			knh_Token_tc(ts[i], &intc);
+			knh_tokens_toArray(ctx, &intc, a, 0);
+			knh_Array_add(ctx, a, UP(TS_RS));
+		}
+		else if(IS_bString(DP(ts[i])->data)) {
+			knh_Array_add(ctx, a, DP(ts[i])->data);
+		}
+		else {
+			knh_Array_add(ctx, a, UP(new_String(ctx, knh_Token_tobytes(ts[i]), NULL)));
 		}
 	}
-	first_tc->e = tc->e;
-	tc->c = tc->e;
 }
 
 /* ------------------------------------------------------------------------ */
 
 static
-void knh_Stmt_add_PAIR(Ctx *ctx, Stmt *stmt_b, knh_tokens_t *tc)
+void knh_Stmt_add_PAIR(Ctx *ctx, Stmt *stmt, knh_tokens_t *tc)
 {
-	knh_tokens_t ctc;
-	knh_tokens_stepFirstColon(tc, &ctc);
-	if(ctc.c == ctc.e || tc->c == tc->e) {
-		DEBUG3("empty!!");
+	int s = tc->c;
+	Token **ts = tc->ts;
+	if(tc->c < tc->e && (SP(ts[tc->e - 1])->tt == TT_COMMA || SP(ts[tc->e - 1])->tt == TT_SEMICOLON)) {
+		tc->e -= 1;
+	}
+
+	if(!(tc->c < tc->e)) {
+		knh_Stmt_perror(ctx, stmt, KMSG_WEMPTY, NULL);
 		return;
 	}
 
-	if(SP(ctc.ts[ctc.e-1])->tt == TT_LABEL) {
-		knh_Stmt_add(ctx, stmt_b, TM(new_TokenCONST(ctx, FL(ctc.ts[ctc.c]), UP(knh_tokens_toString(ctx, &ctc)))));
+	//fprintf(stderr, "NAME s=%d, e=%d\n", s, tc->e);
+
+	if(SP(ts[s])->tt == TT_LABEL) {
+		knh_Stmt_add(ctx, stmt, TM(knh_Token_toCONST(ts[s])));
+		s = tc->c + 1;
+	}
+	else if(DP(ts[s])->tt_next == TT_COLON) {
+		knh_Stmt_add(ctx, stmt, TM(knh_Token_toCONST(ts[s])));
+		s = tc->c + 2;
 	}
 	else {
-		knh_Stmt_add(ctx, stmt_b, new_TermEXPR(ctx, &ctc, KNH_RVALUE));
+		//fprintf(stderr, "tt='%s'\n", knh_token_tochar(SP(ts[s])->tt));
+		knh_Stmt_perror(ctx, stmt, KMSG_WEMPTY, ts[s]);
+		return;
 	}
-	{
-		int c = knh_tokens_count(tc, TT_COMMA);
-		if(c == 0) {
-			knh_Stmt_add(ctx, stmt_b, new_TermEXPR(ctx, tc, KNH_RVALUE));
+
+	//fprintf(stderr, "VALUE s=%d, e=%d\n", s, tc->e);
+	if(s == tc->e) {
+		knh_Stmt_perror(ctx, stmt, KMSG_WEMPTY, ts[s-1]);
+		knh_Stmt_add(ctx, stmt, TM(new_TokenCONST(ctx, UP(ts[s]), KNH_NULL)));
+		return;
+	}
+
+	if(s + 1 == tc->e) {
+		knh_token_t tt = SP(ts[s])->tt;
+		if(tt == TT_NUM || tt == TT_BRACE || tt == TT_BRANCET || tt == TT_PARENTHESIS) {
+			tc->c = s;
+			tc->e = tc->e;
+			knh_Stmt_add(ctx, stmt, new_TermEXPR(ctx, tc, KNH_RVALUE));
+			return ;
 		}
-		else {
-			Stmt *lstmt = new_Stmt(ctx, 0, STT_NEW);
-			knh_Stmt_add(ctx, lstmt, TM(new_TokenMN(ctx, FL(tc->ts[tc->c]), METHODN_new__init)));
-			knh_Stmt_add(ctx, lstmt, TM(new_TokenCID(ctx, FL(tc->ts[tc->c]), CLASS_Array)));
-			knh_Stmt_add_EXPRs(ctx, lstmt, tc);
-			knh_Stmt_add(ctx, stmt_b, TM(lstmt));
+		if(!IS_bString(DP(ts[s])->data)) {
+			KNH_SETv(ctx, DP(ts[s])->data, new_String(ctx, knh_Token_tobytes(ts[s]), NULL));
 		}
+		knh_Stmt_add(ctx, stmt, TM(knh_Token_toCONST(ts[s])));
+		return;
+	}
+
+	/* friend: Person {} */
+	if(s + 2 == tc->e && SP(ts[s])->tt == TT_TYPEN && DP(ts[s])->tt_next == TT_BRACE) {
+		tc->c = s;
+		tc->e = s + 2;
+		knh_Stmt_add(ctx, stmt, new_TermEXPR(ctx, tc, KNH_RVALUE));
+		return ;
+	}
+
+	int i, allnum = 1;
+	for(i = s; i < tc->e; i++) {
+		if(SP(ts[i])->tt == TT_COMMA) continue;
+		if(SP(ts[i])->tt != TT_NUM) {
+			allnum = 0;
+			break;
+		}
+	}
+	if(allnum) { /* size: 10pt 20pt */
+		knh_tokens_t numtc = *tc;
+		Stmt *newstmt = new_Stmt(ctx, 0, STT_NEW);
+		knh_Stmt_add(ctx, newstmt, TM(new_TokenMN(ctx, FL(stmt), METHODN_new__init)));
+		knh_Stmt_add(ctx, newstmt, TM(new_TokenCID(ctx, FL(stmt), CLASS_Array)));
+		for(i = s; i < tc->e; i++) {
+			if(SP(ts[i])->tt == TT_COMMA) continue;
+			if(SP(ts[i])->tt == TT_NUM) {
+				numtc.c = i; numtc.e = i + 1;
+				knh_Stmt_add(ctx, newstmt, new_TermEXPR(ctx, &numtc, KNH_RVALUE));
+			}
+		}
+		knh_Stmt_add(ctx, stmt, TM(newstmt));
+		return;
+	}
+	else {  /* func: int func(int n); */
+		Array *a = new_Array(ctx, CLASS_String, 0);
+		knh_Stmt_add(ctx, stmt, TM(new_TokenCONST(ctx, FL(stmt), UP(a))));
+		tc->c = s;
+		knh_tokens_toArray(ctx, tc, a, 1);
 	}
 }
 
@@ -763,46 +834,38 @@ void knh_Stmt_add_PAIR(Ctx *ctx, Stmt *stmt_b, knh_tokens_t *tc)
 static
 Stmt *new_StmtDICTMAP(Ctx *ctx, Token *ctk, Token *tk)
 {
-	int c;
 	knh_tokens_t tc;
 	knh_Token_tc(tk, &tc);
-	c = knh_tokens_count(&tc, TT_SEMICOLON);
-	if(c == 0) {
-		Stmt *stmt_b = new_Stmt(ctx, 0, STT_NEW);
-		knh_Stmt_add(ctx, stmt_b, TM(new_TokenMN(ctx, FL(tk), METHODN_new__init)));
-		if(IS_NULL(ctk)) {
-			knh_Stmt_add(ctx, stmt_b, TM(new_TokenCID(ctx, FL(tk), CLASS_DictMap)));
-		}
-		else {
-			knh_Stmt_add(ctx, stmt_b, TM(ctk));
-		}
-		while(tc.c < tc.e) {
-			knh_tokens_t sub_tc;
-			knh_tokens_stepFirstComma(ctx, &tc, &sub_tc);
-			if(sub_tc.c < sub_tc.e) {
-				knh_Stmt_add_PAIR(ctx, stmt_b, &sub_tc);
-			}
-		}
-		return stmt_b;
+	Stmt *stmt = new_Stmt(ctx, 0, STT_NEW);
+	knh_Stmt_add(ctx, stmt, TM(new_TokenMN(ctx, FL(tk), METHODN_new__init)));
+	if(IS_NULL(ctk)) {
+		knh_Stmt_add(ctx, stmt, TM(new_TokenCID(ctx, FL(tk), CLASS_DictMap)));
 	}
 	else {
-		Stmt *stmt_b = new_Stmt(ctx, 0, STT_NEW);
-		knh_Stmt_add(ctx, stmt_b, TM(new_TokenMN(ctx, FL(tk), METHODN_new__init)));
-		if(IS_NULL(ctk)) {
-			knh_Stmt_add(ctx, stmt_b, TM(new_TokenCID(ctx, FL(tk), CLASS_DictMap)));
-		}
-		else {
-			knh_Stmt_add(ctx, stmt_b, TM(ctk));
-		}
-		while(tc.c < tc.e) {
-			knh_tokens_t sub_tc;
-			knh_tokens_stepFirstEXPR(ctx, &tc, TT_SEMICOLON, &sub_tc);
-			if(sub_tc.c < sub_tc.e) {
-				knh_Stmt_add_PAIR(ctx, stmt_b, &sub_tc);
-			}
-		}
-		return stmt_b;
+		knh_Stmt_add(ctx, stmt, TM(ctk));
 	}
+
+	int i;
+	knh_tokens_t pair_tc = tc;
+	for(i = 1; i < tc.e; i++) {
+		if(SP(tc.ts[i])->tt == TT_LABEL) {
+			pair_tc.e = i;
+			knh_Stmt_add_PAIR(ctx, stmt, &pair_tc);
+			pair_tc.c = i;
+			continue;
+		}
+		if(DP(tc.ts[i])->tt_next == TT_COLON &&
+			(SP(tc.ts[i])->tt == TT_STR || SP(tc.ts[i])->tt == TT_TSTR || SP(tc.ts[i])->tt == TT_NUM)) {
+			pair_tc.e = i;
+			knh_Stmt_add_PAIR(ctx, stmt, &pair_tc);
+			pair_tc.c = i;
+		}
+	}
+	if(tc.c + 1 < tc.e) {
+		pair_tc.e = tc.e;
+		knh_Stmt_add_PAIR(ctx, stmt, &pair_tc);
+	}
+	return stmt;
 }
 
 /* ======================================================================== */
@@ -823,43 +886,6 @@ Stmt *new_StmtPROPN(Ctx *ctx, Token *tk, int lr)
 	return stmt;
 }
 
-///* ======================================================================== */
-///* [print] */
-//
-//void knh_Stmt_add_printname(Ctx *ctx, Stmt *stmt_b, Token **ts, int s, int e)
-//{
-//	//DEBUG("s=%d, e=%d", s, e);
-//	if(s + 1 == e) {
-//		knh_wbuf_t cb = knh_Context_wbuf(ctx);
-//		if(DP(stmt_b)->size > 2) {
-//			knh_putc(ctx, cb.w, ',');
-//			knh_putc(ctx, cb.w, ' ');
-//		}
-//		int pn = 0;
-//		switch(ts[s]->tt) {
-//		case TT_NAME:
-//			knh_write(ctx, cb.w, knh_Token_tobytes(ts[s]));
-//			pn = 1;
-//			break;
-//		case TT_PROPN:
-//			knh_putc(ctx, cb.w, '$');
-//			knh_write(ctx, cb.w, knh_Token_tobytes(ts[s]));
-//			pn = 1;
-//			break;
-//		}
-//		if(pn == 1) {
-////			knh_putc(ctx, cb.w, ' ');
-//			knh_putc(ctx, cb.w, '=');
-//			knh_Stmt_add(ctx, stmt_b, new_Token__RAW(ctx, 0, ts[s], new_StringX__buffer(ctx, CLASS_String, cb)));
-//		}
-//		else {
-//			knh_wbuf_clear(cb);
-//		}
-//	}
-//	;
-//}
-//
-
 /* ======================================================================== */
 /* [VALUE] */
 
@@ -928,9 +954,9 @@ Stmt* new_StmtFUNCEXPR(Ctx *ctx, knh_tokens_t *tc)
 			return new_StmtMETHOD(ctx, tc);
 		}
 		if(DP(ts[c])->tt_next == TT_NAME && knh_Token_isVARN(ts[c+1])) {  /* @TEST int name; */
-			Token *tk_TYPEN = ts[c];
-			tc->c += 1;
-			return new_StmtDECL(ctx, tk_TYPEN, tc, -2);
+			Token *tk_TYPEN = ts[c]; tc->c += 1;
+			knh_tokens_t expr_tc = knh_tokens_splitSTMT(ctx, tc);
+			return new_StmtDECL(ctx, tk_TYPEN, &expr_tc, -2);
 		}
 		if(DP(ts[c])->tt_next == TT_PARENTHESIS && DP(ts[c+1])->tt_next== TT_BRACE) { /* @TEST int (n) {...} */
 			return new_StmtMETHOD(ctx, tc);
@@ -968,12 +994,7 @@ Stmt *new_StmtINSTMT(Ctx *ctx, Token *tk)
 #if defined(KNH_DBGMODE2)
 		prev = tc.c;
 #endif
-		if(first_stmt == NULL) {
-			Stmt *stmt = new_StmtSTMT1(ctx, &tc);
-			if(stmt != NULL) first_stmt = stmt;
-		}else{
-			knh_Stmt_tail_append(ctx, first_stmt, new_StmtSTMT1(ctx, &tc));
-		}
+		first_stmt = knh_StmtNULL_tail_append(ctx, first_stmt, new_StmtSTMT1(ctx, &tc));
 #if defined(KNH_DBGMODE2)
 		if(prev == tc.c) { /* infinate loop */
 			DBG2_P("Infinate loop? prev = %d, c = %d, e = %d", prev, tc.c, tc.e);
@@ -1025,17 +1046,22 @@ Term *new_TermPEXPR(Ctx *ctx, Token *tk)
 
 /* ------------------------------------------------------------------------ */
 
-void knh_Stmt_add_PEXPR(Ctx *ctx, Stmt *stmt_b, knh_tokens_t *tc)
+void knh_Stmt_add_PEXPR(Ctx *ctx, Stmt *stmt, knh_tokens_t *tc)
 {
-	if(SP(stmt_b)->stt == STT_ERR) return;
+	if(SP(stmt)->stt == STT_ERR) return;
+	if(tc->c < tc->e) {
+		if(SP(tc->ts[tc->c])->tt == TT_WHILE) {
+			tc->c += 1;  /* @ADHOC do {} while() */
+		}
+	}
 	if(tc->c < tc->e) {
 		if(SP(tc->ts[tc->c])->tt == TT_PARENTHESIS) {
-			knh_Stmt_add(ctx, stmt_b, new_TermPEXPR(ctx, tc->ts[tc->c]));
+			knh_Stmt_add(ctx, stmt, new_TermPEXPR(ctx, tc->ts[tc->c]));
 			tc->c += 1;
 			return ;
 		}
 	}
-	knh_Stmt_perror(ctx, stmt_b, tc, KMSG_TPEXPR);
+	knh_Stmt_tokens_perror(ctx, stmt, tc, KMSG_TPEXPR);
 }
 
 /* ======================================================================== */
@@ -1091,6 +1117,7 @@ Stmt *new_StmtOP(Ctx *ctx, knh_tokens_t *tc, Token *op)
 		case TT_QUESTION: goto L_TRINARY;
 		case TT_NEXT: case TT_PREV: goto L_NEXT;
 	}
+
 	{
 		Stmt *stmt;
 		if(tc->ts[tc->c] == op) {
@@ -1106,8 +1133,7 @@ Stmt *new_StmtOP(Ctx *ctx, knh_tokens_t *tc, Token *op)
 		}
 
 		while(tc->c < tc->e) {
-			knh_tokens_t op_tc;
-			knh_tokens_stepFirstEXPR(ctx, tc, SP(op)->tt, &op_tc);
+			knh_tokens_t op_tc = knh_tokens_splitEXPR(ctx, tc, SP(op)->tt);
 			if(SP(op)->tt != TT_FMT) {
 				knh_Stmt_add(ctx, stmt, new_TermEXPR(ctx, &op_tc, KNH_RVALUE));
 			}
@@ -1129,35 +1155,34 @@ Stmt *new_StmtOP(Ctx *ctx, knh_tokens_t *tc, Token *op)
 
 	}
 	L_TRINARY:; {
-		Stmt *stmt_b = new_Stmt(ctx, 0, STT_TRINARY);
-		knh_tokens_t expr_tc;
-		knh_tokens_stepFirstEXPR(ctx, tc, SP(op)->tt, &expr_tc);
-		knh_Stmt_add(ctx, stmt_b, new_TermEXPR(ctx, &expr_tc, KNH_RVALUE));
-		knh_tokens_stepFirstColon(tc, &expr_tc);
+		Stmt *stmt = new_Stmt(ctx, 0, STT_TRINARY);
+		knh_tokens_t expr_tc = knh_tokens_splitEXPR(ctx, tc, SP(op)->tt);
+		knh_Stmt_add(ctx, stmt, new_TermEXPR(ctx, &expr_tc, KNH_RVALUE));
+		expr_tc = knh_tokens_splitEXPR(ctx, tc, TT_COLON);
 		if(expr_tc.c < expr_tc.e && tc->c < tc-> e) {
-			knh_Stmt_add(ctx, stmt_b, new_TermEXPR(ctx, &expr_tc, KNH_RVALUE));
-			knh_Stmt_add(ctx, stmt_b, new_TermEXPR(ctx, tc, KNH_RVALUE));
+			knh_Stmt_add(ctx, stmt, new_TermEXPR(ctx, &expr_tc, KNH_RVALUE));
+			knh_Stmt_add(ctx, stmt, new_TermEXPR(ctx, tc, KNH_RVALUE));
 		}
 		else {
-			knh_Stmt_perror(ctx, stmt_b, &expr_tc, KMSG_ETRINARY);
+			knh_Stmt_tokens_perror(ctx, stmt, &expr_tc, KMSG_ESYNTAX);
 		}
-		return stmt_b;
+		return stmt;
 	}
+
 	L_NEXT:; {
-		knh_tokens_t ltc, rtc;
 		if(tc->ts[tc->c] == op) {
 			tc->c += 1;
 		}
-		knh_tokens_stepFirstEXPR(ctx, tc, SP(op)->tt, &ltc);
-		rtc = ltc;
+		knh_tokens_t ltc = knh_tokens_splitEXPR(ctx, tc, SP(op)->tt);
+		knh_tokens_t rtc = ltc;
 		{
 			Term *lval = new_TermEXPR(ctx, &ltc, KNH_LVALUE);
 			if(IS_Token(lval)) {
 				if(knh_Token_isLVALUE((Token*)lval)) {
-					Stmt *stmt_b = new_Stmt(ctx, 0, STT_LET);
-					knh_Stmt_add(ctx, stmt_b, lval);
-					knh_Stmt_add(ctx, stmt_b, TM(new_StmtNEXT(ctx, op, &rtc)));
-					return stmt_b;
+					Stmt *stmt = new_Stmt(ctx, 0, STT_LET);
+					knh_Stmt_add(ctx, stmt, lval);
+					knh_Stmt_add(ctx, stmt, TM(new_StmtNEXT(ctx, op, &rtc)));
+					return stmt;
 				}
 				else {
 					knh_Token_perror(ctx, op, KMSG_ENEXTPREV);
@@ -1165,13 +1190,13 @@ Stmt *new_StmtOP(Ctx *ctx, knh_tokens_t *tc, Token *op)
 				}
 			}
 			else {
-				Stmt *stmt_b = (Stmt*)lval;
-				KNH_ASSERT(IS_Stmt(stmt_b));
-				knh_Stmt_toLVALUE(ctx, stmt_b, KMSG_ENEXTPREV);
-				if(SP(stmt_b)->stt != STT_ERR) {
-					knh_Stmt_add(ctx, stmt_b, TM(new_StmtNEXT(ctx, op, &rtc)));
+				Stmt *stmt = (Stmt*)lval;
+				KNH_ASSERT(IS_Stmt(stmt));
+				knh_Stmt_toLVALUE(ctx, stmt, KMSG_ENEXTPREV);
+				if(SP(stmt)->stt != STT_ERR) {
+					knh_Stmt_add(ctx, stmt, TM(new_StmtNEXT(ctx, op, &rtc)));
 				}
-				return stmt_b;
+				return stmt;
 			}
 		}
 	}
@@ -1232,7 +1257,7 @@ Term *new_TermEXPR(Ctx *ctx, knh_tokens_t *tc, int lr)
 	if(!(oc < e)) {
 		DBG2_P("tc->c=%d, tc->e=%d", (int)oc, (int)e);
 		KNH_ASSERT(e > 0);
-		KNH_ASSERT(oc < e);
+		//KNH_ASSERT(oc < e);
 		Token *tke = new_Token(ctx, 0, SP(tc->ts[e-1])->fileid, SP(tc->ts[e-1])->line, TT_ERR);
 		knh_perror(ctx, SP(tc->ts[e-1])->fileid, SP(tc->ts[e-1])->line, KMSG_ESYNTAX, NULL);
 		return TM(tke);
@@ -1302,7 +1327,21 @@ Term *new_TermEXPR(Ctx *ctx, knh_tokens_t *tc, int lr)
 				}
 				if(DP(ts[oc+1])->tt_next == TT_BRANCET) {   /* @TEST new Int[10] */
 					stmt = new_Stmt(ctx, 0, STT_NEW);
+#ifdef METHODN_new__array2D
+					knh_tokens_t cma_tc;
+					knh_Token_tc(ts[oc+2], &cma_tc);
+					if(knh_tokens_count(&cma_tc, TT_COMMA) == 1) {
+						knh_Stmt_add(ctx, stmt, TM(new_TokenMN(ctx, FL(ts[oc]), METHODN_new__array2D)));
+					}
+					else if(knh_tokens_count(&cma_tc, TT_COMMA) == 2) {
+						knh_Stmt_add(ctx, stmt, TM(new_TokenMN(ctx, FL(ts[oc]), METHODN_new__array3D)));
+					}
+					else {
+						knh_Stmt_add(ctx, stmt, TM(new_TokenMN(ctx, FL(ts[oc]), METHODN_new__array)));
+					}
+#else
 					knh_Stmt_add(ctx, stmt, TM(new_TokenMN(ctx, FL(ts[oc]), METHODN_new__array)));
+#endif
 					knh_Stmt_add(ctx, stmt, TM(ts[oc+1]));
 					knh_Token_setArrayType(ts[oc+1], 1);
 					pc = oc + 2;
@@ -1311,7 +1350,7 @@ Term *new_TermEXPR(Ctx *ctx, knh_tokens_t *tc, int lr)
 					goto L_PARAM;
 				}
 				tc->c = oc + 2;
-				knh_tokens_perror(ctx, tc, KMSG_TARGS);  /* @TEST new Int j */
+				knh_Token_perror(ctx, ts[oc+2], KMSG_ESYNTAX);
 				return TM(new_StmtERR(ctx, tc));
 			}
 			else if(DP(ts[oc])->tt_next == TT_NAME && ISB(knh_Token_tobytes(ts[oc+1]), "byte") && DP(ts[oc+1])->tt_next == TT_BRANCET) {
@@ -1348,7 +1387,7 @@ Term *new_TermEXPR(Ctx *ctx, knh_tokens_t *tc, int lr)
 	if(SP(ts[oc])->tt == TT_MT) {  /* @TEST %s(a) */
 		if(DP(ts[oc])->tt_next != TT_PARENTHESIS) {
 			tc->c = oc + 1;
-			knh_tokens_perror(ctx, tc, KMSG_TARGS);
+			knh_Token_perror(ctx, ts[oc], KMSG_ESYNTAX);
 			return TM(new_StmtERR(ctx, tc));
 		}
 		stmt = new_Stmt(ctx, 0, STT_MT);
@@ -1459,24 +1498,6 @@ Term *new_TermEXPR(Ctx *ctx, knh_tokens_t *tc, int lr)
 	return TM(stmt);
 }
 
-/* ======================================================================== */
-/* [(expr,expr,expr)] */
-
-void knh_Stmt_add_EXPRs(Ctx *ctx, Stmt *stmt_b, knh_tokens_t *tc)
-{
-	while(tc->c < tc->e) {
-		knh_tokens_t sub_tc;
-		Token *tk = tc->ts[tc->c];
-		if(/* SP(tk)->tt == TT_SEMICOLON || */ knh_Token_isBOL(tk)) {
-			return ;
-		}
-		//DEBUG3_PTC("tc", tc);
-		knh_tokens_stepFirstComma(ctx, tc, &sub_tc);
-		if(sub_tc.c < sub_tc.e) {
-			knh_Stmt_add(ctx, stmt_b, new_TermEXPR(ctx, &sub_tc, KNH_RVALUE));
-		}
-	}
-}
 
 /* ------------------------------------------------------------------------ */
 
