@@ -216,7 +216,7 @@ void
 knh_Stmt_add_STEXT(Ctx *ctx, Stmt *o, knh_tokens_t *tc, knh_bytes_t stext)
 {
 	if(SP(o)->stt == STT_ERR) return;
-	knh_Stmt_add(ctx, o, TM(new_Token__text(ctx, knh_tokens_curToken(tc), stext)));
+	knh_Stmt_add(ctx, o, TM(new_Token__text(ctx, FL(o), stext)));
 }
 
 /* ------------------------------------------------------------------------ */
@@ -261,12 +261,12 @@ knh_Stmt_add_PRAGMA(Ctx *ctx, Stmt *o, knh_tokens_t *tc)
 					tc->c += 1;
 					break;
 				default:
-					knh_Stmt_perror(ctx, o, tc, KMSG_TARGS);
+					knh_Stmt_tokens_perror(ctx, o, tc, KMSG_TARGS);
 					return;
 			}
 		}
 		else {
-			knh_Stmt_perror(ctx, o, tc, KMSG_TARGS);
+			knh_Stmt_tokens_perror(ctx, o, tc, KMSG_TARGS);
 		}
 	}
 	else {
@@ -363,7 +363,7 @@ void knh_Stmt_add_CFUNCN(Ctx *ctx, Stmt *o, knh_tokens_t *tc)
 		tc->c += 1;
 	}
 	else {
-		knh_Stmt_perror(ctx, o, tc, KMSG_TCMETHODN);
+		knh_Stmt_tokens_perror(ctx, o, tc, KMSG_TCMETHODN);
 	}
 }
 
@@ -552,11 +552,10 @@ knh_Stmt_add_ELSE(Ctx *ctx, Stmt *o, knh_tokens_t *tc)
 void
 knh_Stmt_add_CLASSTNs(Ctx *ctx, Stmt *o, knh_tokens_t *tc)
 {
-	knh_tokens_t stmt_tc, comma_tc;
 	if(SP(o)->stt == STT_ERR) return;
-	knh_tokens_stepFirstStmt(ctx, tc, &stmt_tc, 1);
+	knh_tokens_t stmt_tc = knh_tokens_splitSTMT(ctx, tc);
 	while(stmt_tc.c < stmt_tc.e) {
-		knh_tokens_stepFirstComma(ctx, &stmt_tc, &comma_tc);
+		knh_tokens_t comma_tc = knh_tokens_splitEXPR(ctx, &stmt_tc, TT_COMMA);
 		if(comma_tc.c < comma_tc.e) {
 			knh_Stmt_add_CLASSTN(ctx, o, &comma_tc);
 			knh_tokens_ignore(ctx, &comma_tc);
@@ -564,7 +563,6 @@ knh_Stmt_add_CLASSTNs(Ctx *ctx, Stmt *o, knh_tokens_t *tc)
 	}
 }
 
-
 /* ======================================================================== */
 /* [stmt3] */
 
@@ -574,8 +572,7 @@ knh_Stmt_add_PSTMT3(Ctx *ctx, Stmt *o, knh_tokens_t *tc)
 	Token *tk = tc->ts[tc->c];
 	if(SP(o)->stt == STT_ERR) return;
 	if(tc->c < tc->e && SP(tk)->tt == TT_PARENTHESIS) {
-		knh_tokens_t ptc, stmt_tc;
-		knh_Token_tokens_add(ctx, tk, new_Token__text(ctx, tk, STEXT(";")));
+		knh_tokens_t ptc;
 		knh_Token_tc(tc->ts[tc->c], &ptc);
 		if(knh_tokens_count(&ptc, TT_COLON) > 0) {
 			SP(o)->stt = STT_FOREACH;
@@ -584,41 +581,35 @@ knh_Stmt_add_PSTMT3(Ctx *ctx, Stmt *o, knh_tokens_t *tc)
 		}
 		else {
 			int c = knh_tokens_count(&ptc, TT_SEMICOLON);
-			if(c != 3) {
-				knh_Stmt_perror(ctx, o, tc, KMSG_EPSTMT3);
+			if(c != 2) {
+				knh_Stmt_tokens_perror(ctx, o, tc, KMSG_EPSTMT3);
 				return ;
 			}
 			else {
 				/* for(FIRST;second;third) */
-				knh_tokens_stepFirstStmt(ctx, &ptc, &stmt_tc, 1);
-				//DEBUG3_STC("ptc", ptc);
-				//DEBUG3_STC("stmt_tc", stmt_tc);
-				if(stmt_tc.c == stmt_tc.e) {
+				knh_tokens_t stmt_tc = knh_tokens_splitEXPR(ctx, &ptc, TT_SEMICOLON);
+				if(stmt_tc.c < stmt_tc.e) {
+//					KNH_ASSERT(SP(stmt_tc.ts[stmt_tc.e])->tt == TT_SEMICOLON);
+//					stmt_tc.e++;
+					knh_Stmt_add(ctx, o, TM(new_StmtLETEXPR(ctx, &stmt_tc)));
+				}
+				else {
 					knh_Stmt_add(ctx, o, TM(new_StmtDONE(ctx)));
 				}
-				else {
-					KNH_ASSERT(SP(stmt_tc.ts[stmt_tc.e])->tt == TT_SEMICOLON);
-					stmt_tc.e++;
-					knh_Stmt_add(ctx, o, TM(new_StmtFUNCEXPR(ctx, &stmt_tc)));
-				}
-				knh_tokens_stepFirstStmt(ctx, &ptc, &stmt_tc, 1);
-				//DEBUG3_STC("ptc2", ptc);
-				//DEBUG3_STC("stmt_tc2", stmt_tc);
+				stmt_tc = knh_tokens_splitEXPR(ctx, &ptc, TT_SEMICOLON);
 				/* for(first;SECODN;third) */
-				if(stmt_tc.c == stmt_tc.e) {
-					knh_Stmt_add(ctx, o, TM(new_TokenCONST(ctx, FL(tc->ts[tc->c]), KNH_TRUE)));
-				}
-				else {
+				if(stmt_tc.c < stmt_tc.e) {
 					knh_Stmt_add(ctx, o, new_TermEXPR(ctx, &stmt_tc, KNH_RVALUE));
 				}
+				else {
+					knh_Stmt_add(ctx, o, TM(new_TokenCONST(ctx, FL(tc->ts[tc->c]), KNH_TRUE)));
+				}
 				/* for(first;second;THIRD) */
-				if(ptc.c == ptc.e-1) {
-					knh_Stmt_add(ctx, o, TM(new_StmtDONE(ctx)));
+				if(ptc.c < ptc.e) {
+					knh_Stmt_add(ctx, o, TM(new_StmtLETEXPR(ctx, &ptc)));
 				}
 				else {
-					KNH_ASSERT(SP(ptc.ts[ptc.e-1])->tt == TT_SEMICOLON);
-					ptc.e++;
-					knh_Stmt_add(ctx, o, TM(new_StmtLETEXPR(ctx, &ptc)));
+					knh_Stmt_add(ctx, o, TM(new_StmtDONE(ctx)));
 				}
 			}
 		}
@@ -626,7 +617,7 @@ knh_Stmt_add_PSTMT3(Ctx *ctx, Stmt *o, knh_tokens_t *tc)
 		knh_Stmt_add_STMT1(ctx, o, tc);
 	}
 	else {
-		knh_Stmt_perror(ctx, o, tc, KMSG_EPSTMT3);
+		knh_Stmt_tokens_perror(ctx, o, tc, KMSG_EPSTMT3);
 	}
 }
 
@@ -660,14 +651,14 @@ Stmt *new_StmtCATCH(Ctx *ctx, knh_tokens_t *tc)
 		tc->c += 1;
 	}
 	else {
-		knh_Stmt_perror(ctx, o, tc, KMSG_TCATCHPARAM);
+		knh_Stmt_tokens_perror(ctx, o, tc, KMSG_TCATCHPARAM);
 		return o;
 	}
 	knh_Stmt_add_STMT1(ctx, o, tc);
 
 	if(tc->c < tc->e && SP(tc->ts[tc->c])->tt == TT_CATCH) {
 		tc->c += 1;
-		knh_Stmt_tail_append(ctx, o, new_StmtCATCH(ctx, tc));
+		knh_StmtNULL_tail_append(ctx, o, new_StmtCATCH(ctx, tc));
 	}
 	return o;
 }
@@ -722,7 +713,7 @@ knh_Stmt_add_CMETHOD(Ctx *ctx, Stmt *o, knh_tokens_t *tc)
 			goto L_PARAMS;
 		}
 	}
-	knh_Stmt_perror(ctx, o, tc, KMSG_TMETHODN);
+	knh_Stmt_tokens_perror(ctx, o, tc, KMSG_TMETHODN);
 	return;
 
 	L_PARAMS:;
@@ -752,7 +743,7 @@ void knh_Stmt_add_TYPEVARN(Ctx *ctx, Stmt *o, knh_tokens_t *tc)
 			return;
 		}
 	}
-	knh_Stmt_perror(ctx, o, tc, KMSG_TTYPEN);
+	knh_Stmt_tokens_perror(ctx, o, tc, KMSG_TTYPEN);
 }
 
 /* ------------------------------------------------------------------------ */
@@ -771,8 +762,7 @@ knh_Stmt_add_PARAMs(Ctx *ctx, Stmt *o, knh_tokens_t *tc)
 		}
 		while(param_tc.c <param_tc.e) {
 			Stmt *o2 = new_Stmt(ctx, 0, STT_DECL);
-			knh_tokens_t ptc;
-			knh_tokens_stepFirstComma(ctx, &param_tc, &ptc);
+			knh_tokens_t ptc = knh_tokens_splitEXPR(ctx, &param_tc, TT_COMMA);
 			knh_StmtMETA_add(ctx, o2, &ptc);
 			knh_Stmt_add_TYPEVARN(ctx, o2, &ptc);
 			if(ptc.c + 1 < ptc.e && SP(ptc.ts[ptc.c])->tt == TT_LET) {
@@ -783,18 +773,13 @@ knh_Stmt_add_PARAMs(Ctx *ctx, Stmt *o, knh_tokens_t *tc)
 				knh_tokens_ignore(ctx, &ptc);
 				knh_Stmt_add_ASIS(ctx, o2);
 			}
-			if(stmt != NULL) {
-				knh_Stmt_tail_append(ctx, stmt, o2);
-			}
-			else {
-				stmt = o2;
-			}
+			stmt =knh_StmtNULL_tail_append(ctx, stmt, o2);
 		}
 		knh_Stmt_add(ctx, o, TM(stmt));
 		return ;
 	}
 	else {
-		knh_Stmt_perror(ctx, o, tc, KMSG_TARGS);
+		knh_Stmt_tokens_perror(ctx, o, tc, KMSG_TARGS);
 	}
 }
 
@@ -872,7 +857,7 @@ knh_Stmt_add_PEACH(Ctx *ctx, Stmt *o, knh_tokens_t *tc)
 			}
 		}
 		DEBUG3("NO FROM!!");
-		knh_Stmt_perror(ctx, o, tc, KMSG_EPEACH);
+		knh_Stmt_tokens_perror(ctx, o, tc, KMSG_EPEACH);
 		return;
 
 		L_FROM:;
@@ -891,7 +876,7 @@ knh_Stmt_add_PEACH(Ctx *ctx, Stmt *o, knh_tokens_t *tc)
 		}
 		if(!(from_tc.c < from_tc.e)) {
 			DEBUG3("NO FROM!! c=%d, e=%d", from_tc.c, from_tc.e);
-			knh_Stmt_perror(ctx, o, tc, KMSG_EPEACH);
+			knh_Stmt_tokens_perror(ctx, o, tc, KMSG_EPEACH);
 			return;
 		}
 
@@ -909,7 +894,7 @@ knh_Stmt_add_PEACH(Ctx *ctx, Stmt *o, knh_tokens_t *tc)
 		knh_Stmt_add_STMT1(ctx, o, tc);
 	}
 	else {
-		knh_Stmt_perror(ctx, o, tc, KMSG_EPEACH);
+		knh_Stmt_tokens_perror(ctx, o, tc, KMSG_EPEACH);
 	}
 }
 /* ------------------------------------------------------------------------ */
