@@ -53,7 +53,9 @@ typedef struct {  /* @data */
 } knh_tokenctx_t;
 
 static knh_bool_t knh_Bytes_isTripleQuote(Bytes *b, knh_int_t ch);
-static knh_tokenctx_t *knh_pstack_back(Ctx *ctx, knh_tokenctx_t *ps, knh_tokenctx_t *psp, int indent);
+static knh_tokenctx_t *knh_pstack_backindent(Ctx *ctx, knh_tokenctx_t *ps, knh_tokenctx_t *psp, int indent);
+static knh_tokenctx_t *knh_pstack_backstmt(Ctx *ctx, knh_tokenctx_t *ps, knh_tokenctx_t *psp);
+
 static Token *new_Token__buffer(Ctx *ctx, knh_token_t tt, knh_wbuf_t tbuf, InputStream *in);
 static void knh_Token_padd(Ctx *ctx, knh_tokenctx_t *tkctx, Token *tk);
 
@@ -95,7 +97,7 @@ void knh_Token_parse(Ctx *ctx, Token *tk, InputStream *in, int join)
 
 	if(indent < tkctx->indent) {
 		knh_perror(ctx, DP(in)->fileid, DP(in)->line, KMSG_WINDENT, NULL);
-		tkctx = knh_pstack_back(ctx, tkctx_stack, tkctx, indent);
+		tkctx = knh_pstack_backindent(ctx, tkctx_stack, tkctx, indent);
 	}
 	tkctx->curindent = indent;
 	goto MAIN_PART_INLOOP;
@@ -119,9 +121,9 @@ void knh_Token_parse(Ctx *ctx, Token *tk, InputStream *in, int join)
 		case '{':
 			knh_Token_add_space(ctx, tkctx, tbuf, in, join);
 			if(SP(tkctx->tk)->tt == TT_PARENTHESIS && IS_NOTNULL(DP(tkctx->tk)->data)) {
-				DBG2_P("@Probabry forget ')': indent=%d, psp->indent=%d", indent, tkctx->indent);
-				knh_perror(ctx, DP(in)->fileid, DP(in)->line, KMSG_EFORGET_PARENTHESIS, "..) {");
-				tkctx = knh_pstack_back(ctx, tkctx_stack, tkctx, indent);
+				//DBG2_P("@Probabry forget ')': indent=%d, psp->indent=%d", indent, tkctx->indent);
+				knh_perror(ctx, DP(in)->fileid, DP(in)->line, KMSG_EFORGET_PARENTHESIS, ")");
+				tkctx = knh_pstack_backstmt(ctx, tkctx_stack, tkctx);
 			}
 
 		case '(': 	case '[':
@@ -183,7 +185,20 @@ void knh_Token_parse(Ctx *ctx, Token *tk, InputStream *in, int join)
 			prev = ch;
 			goto QUOTED_PART;
 
-		case ',': case ';':
+		case ';':
+			knh_Token_add_space(ctx, tkctx, tbuf, in, join);
+//			if(SP(tkctx->tk)->tt == TT_PARENTHESIS) {
+//				knh_perror(ctx, DP(in)->fileid, DP(in)->line, KMSG_EFORGET_PARENTHESIS, ")");
+//				tkctx = knh_pstack_backstmt(ctx, tkctx_stack, tkctx);
+//			}
+//			else if(SP(tkctx->tk)->tt == TT_BRANCET) {
+//				knh_perror(ctx, DP(in)->fileid, DP(in)->line, KMSG_EFORGET_PARENTHESIS, "]");
+//				tkctx = knh_pstack_backstmt(ctx, tkctx_stack, tkctx);
+//			}
+			knh_Token_padd(ctx, tkctx, new_Token(ctx, 0, DP(in)->fileid, DP(in)->line, knh_char_totoken(ch)));
+			break;
+
+		case ',':
 			knh_Token_add_space(ctx, tkctx, tbuf, in, join);
 			knh_Token_padd(ctx, tkctx, new_Token(ctx, 0, DP(in)->fileid, DP(in)->line, knh_char_totoken(ch)));
 			break;
@@ -423,24 +438,6 @@ void knh_Token_parse(Ctx *ctx, Token *tk, InputStream *in, int join)
 				knh_Token_add_space(ctx, tkctx, tbuf, in, join);
 				goto MAIN_PART_INLOOP;
 			}
-//
-//			switch(ch) {
-//				case '-': case '|': case '&':
-//					if(prev == '=' || prev == '>' || prev == '<') {
-//						knh_Token_add_space(ctx, psp->tk, tbuf, in);
-//					}
-//					knh_Bytes_putc(ctx, tbuf.ba, ch);
-//					prev = ch;
-//					break;
-//				case '=': case '+': case '<': case '>':
-//				case '.': case '?': case ':':
-//					knh_Bytes_putc(ctx, tbuf.ba, ch);
-//					prev = ch;
-//					break;
-//				default:
-//					knh_Token_add_space(ctx, psp->tk, tbuf, in);
-//					goto MAIN_PART_INLOOP;
-//			}
 		}
 	}
 	goto MAIN_PART_INLOOP; /* EOF */
@@ -830,8 +827,9 @@ Token *new_Token__name(Ctx *ctx, knh_flag_t flag, knh_fileid_t fileid, knh_line_
 }
 
 /* ------------------------------------------------------------------------ */
+
 static
-knh_tokenctx_t *knh_pstack_back(Ctx *ctx, knh_tokenctx_t *ps, knh_tokenctx_t *psp, int indent)
+knh_tokenctx_t *knh_pstack_backindent(Ctx *ctx, knh_tokenctx_t *ps, knh_tokenctx_t *psp, int indent)
 {
 	DEBUG3("@S indent=%d", indent);
 	while(ps < psp) {
@@ -846,6 +844,18 @@ knh_tokenctx_t *knh_pstack_back(Ctx *ctx, knh_tokenctx_t *ps, knh_tokenctx_t *ps
 
 /* ------------------------------------------------------------------------ */
 
+static
+knh_tokenctx_t *knh_pstack_backstmt(Ctx *ctx, knh_tokenctx_t *ps, knh_tokenctx_t *psp)
+{
+	while(ps < psp) {
+		psp--;
+		if(SP(psp->tk)->tt != TT_BRACE) continue;
+	}
+	return psp;
+}
+
+/* ------------------------------------------------------------------------ */
+
 static
 knh_bool_t knh_Bytes_isTripleQuote(Bytes *o, knh_int_t ch)
 {
