@@ -56,13 +56,19 @@ Any* knh_Array_pop(Ctx *ctx, Array *o);
 /* ../class/konoha/knh_Array_api1.c */
 METHOD knh__Array_new(Ctx *ctx, knh_sfp_t *sfp);
 METHOD knh__Array_new__array(Ctx *ctx, knh_sfp_t *sfp);
+METHOD knh__Array_new__array2D(Ctx *ctx, knh_sfp_t *sfp);
+METHOD knh__Array_new__array3D(Ctx *ctx, knh_sfp_t *sfp);
 METHOD knh__Array_new__init(Ctx *ctx, knh_sfp_t *sfp);
 METHOD knh__Array_add(Ctx *ctx, knh_sfp_t *sfp);
 METHOD knh__Array_opLshift(Ctx *ctx, knh_sfp_t *sfp);
 METHOD knh__Array_getSize(Ctx *ctx, knh_sfp_t *sfp);
 METHOD knh__Array_get(Ctx *ctx, knh_sfp_t *sfp);
+METHOD knh__Array_get2D(Ctx *ctx, knh_sfp_t *sfp);
+METHOD knh__Array_get3D(Ctx *ctx, knh_sfp_t *sfp);
 METHOD knh__Array_set(Ctx *ctx, knh_sfp_t *sfp);
-METHOD knh__Array_opFill(Ctx *ctx, knh_sfp_t *sfp);
+METHOD knh__Array_set2D(Ctx *ctx, knh_sfp_t *sfp);
+METHOD knh__Array_set3D(Ctx *ctx, knh_sfp_t *sfp);
+METHOD knh__Array_setAll(Ctx *ctx, knh_sfp_t *sfp);
 METHOD knh__Array_opSubsete(Ctx *ctx, knh_sfp_t *sfp);
 METHOD knh__Array_opSubset(Ctx *ctx, knh_sfp_t *sfp);
 METHOD knh__Array_opOffset(Ctx *ctx, knh_sfp_t *sfp);
@@ -101,6 +107,8 @@ Object** knh_oarray_grow(Ctx *ctx, Object **a);
 Object **knh_oarray_copy(Ctx *ctx, Object **a, size_t s, size_t offset);
 void knh_oarray_traverse(Ctx *ctx, Object **a, f_traverse ftr);
 void knh_oarray_free(Ctx *ctx, knh_Object_t **a);
+/* ../class/konoha/knh_Boolean.c */
+Object *knh_Boolean_fdefault(Ctx *ctx, knh_class_t cid);
 /* ../class/konoha/knh_Boolean_api1.c */
 String* knh_Boolean_String(Ctx *ctx, Boolean *o, Mapper *mpr);
 Boolean* knh_String_Boolean(Ctx *ctx, String *o, Mapper *mpr);
@@ -516,7 +524,6 @@ void knh_LRUHashMap_clear(Ctx *ctx, LRUHashMap *o);
 void knh_Mapper_struct_init(Ctx *ctx, knh_Mapper_struct *b, int init, Object *cs);
 void knh_Mapper_struct_traverse(Ctx *ctx, knh_Mapper_struct *b, f_traverse ftr);
 Mapper* new_Mapper(Ctx *ctx, knh_flag_t flag, knh_class_t scid, knh_class_t tcid, f_mapper fmap, Object *mapdata);
-Object *knh_Mapper_fmapmap(Ctx *ctx, Object *s, Mapper *mpr);
 Mapper* new_MapMap(Ctx *ctx, Mapper *m1, Mapper *m2);
 Object *knh_Mapper_execMap(Ctx *ctx, Mapper *o, Object *sub);
 void knh_Mapper__k(Ctx *ctx, Mapper *o, OutputStream *w, String *m);
@@ -575,8 +582,8 @@ knh_class_t knh_tclass_loadURN(Ctx *ctx, knh_class_t bcid, knh_bytes_t urn);
 knh_type_t knh_NameSpace_tagcid(Ctx *ctx, NameSpace *o, knh_class_t bcid, knh_bytes_t tag);
 void knh_NameSpace_setFuncClass(Ctx *ctx, NameSpace *o, knh_methodn_t mn, knh_class_t c);
 knh_class_t knh_NameSpace_getFuncClass(Ctx *ctx, NameSpace *o, knh_bytes_t funcname);
-void knh_NameSpace_addLocalConst(Ctx *ctx, NameSpace *o, String *name, Object *value);
 knh_bool_t knh_NameSpace_existsLocalConst(Ctx *ctx, NameSpace *o, knh_bytes_t name);
+void knh_NameSpace_addLocalConst(Ctx *ctx, NameSpace *o, String *name, Object *value);
 Object* knh_NameSpace_getLocalConst(Ctx *ctx, NameSpace *o, knh_bytes_t name);
 void knh_NameSpace_loaded(Ctx *ctx, NameSpace *o, knh_fileid_t fileid);
 knh_bool_t knh_NameSpace_isLoaded(Ctx *ctx, NameSpace *o, knh_fileid_t fileid);
@@ -585,6 +592,8 @@ Object *new_Nue(Ctx *ctx, String *msg);
 Object *knh_Nue_fdefault(Ctx *ctx, knh_class_t cid);
 int knh_Nue_compareTo(Nue *o, Nue *o2);
 /* ../class/konoha/knh_Nue_api1.c */
+METHOD knh__Nue_opEq(Ctx *ctx, knh_sfp_t *sfp);
+METHOD knh__Nue_opNeq(Ctx *ctx, knh_sfp_t *sfp);
 void knh_Nue__s(Ctx *ctx, Nue *b, OutputStream *w, String *m);
 void knh_Nue__k(Ctx *ctx, Nue *b, OutputStream *w, String *m);
 /* ../class/konoha/knh_Number.c */
@@ -817,15 +826,16 @@ Stmt *knh_konohac_parse(Ctx *ctx, InputStream *in);
 void knh_konohac_eval(Ctx *ctx, String *nsname, InputStream *in);
 /* ../class/konohac/knh_parser_StmtEXPR.c */
 Stmt* new_StmtLETEXPR(Ctx *ctx, knh_tokens_t *tc);
-void knh_Stmt_add_EXPR(Ctx *ctx, Stmt *stmt_b, knh_tokens_t *tc);
+void knh_Stmt_add_EXPR(Ctx *ctx, Stmt *stmt, knh_tokens_t *tc);
+void knh_Stmt_add_EXPRs(Ctx *ctx, Stmt *stmt_b, knh_tokens_t *tc);
 Token *new_TokenCLASSN(Ctx *ctx, Token *tk, knh_bytes_t name);
 Token *new_TokenCONSTN(Ctx *ctx, Token *tk, knh_bytes_t name);
+void knh_tokens_toArray(Ctx *ctx, knh_tokens_t *tc, Array *a, int ignoreCOMMA);
 Stmt* new_StmtFUNCEXPR(Ctx *ctx, knh_tokens_t *tc);
 Stmt *new_StmtINSTMT(Ctx *ctx, Token *tk);
 Term *new_TermPEXPR(Ctx *ctx, Token *tk);
-void knh_Stmt_add_PEXPR(Ctx *ctx, Stmt *stmt_b, knh_tokens_t *tc);
+void knh_Stmt_add_PEXPR(Ctx *ctx, Stmt *stmt, knh_tokens_t *tc);
 Term *new_TermEXPR(Ctx *ctx, knh_tokens_t *tc, int lr);
-void knh_Stmt_add_EXPRs(Ctx *ctx, Stmt *stmt_b, knh_tokens_t *tc);
 /* ../class/konohac/knh_parser_StmtSTMT.c */
 knh_bool_t knh_Token_isVARN(Token *tk);
 knh_bool_t knh_Token_isNSNAME(Token *tk);
@@ -870,10 +880,11 @@ void knh_perrata(Ctx *ctx, knh_fileid_t fileid, int line, char *oldt, char *newt
 void knh_Token_perrata(Ctx *ctx, Token *o, char *newtoken);
 void knh_perror(Ctx *ctx, knh_fileid_t fileid, int line, int pe, char *msg);
 void knh_Token_perror(Ctx *ctx, Token *o, int pe);
-void knh_tokens_perror(Ctx *ctx, knh_tokens_t *tc, int pe);
-void knh_Stmt_perror(Ctx *ctx, Stmt *b, knh_tokens_t *tc, int pe);
+void knh_Stmt_tokens_perror(Ctx *ctx, Stmt *o, knh_tokens_t *tc, int pe);
 Stmt *new_StmtERR(Ctx *ctx, knh_tokens_t *tc);
+void knh_Stmt_perror(Ctx *ctx, Stmt *o, int pe, Token *tk);
 void knh_Compiler_perror(Ctx *ctx, Compiler *cpr, int pe, char *msg);
+void knh_Compiler_assert(Ctx *ctx, Compiler *cpr, int c);
 /* ../class/konohac/knh_Stmt.c */
 void knh_Stmt_struct_init(Ctx *ctx, knh_Stmt_struct *b, int init, Object *cs);
 void knh_Stmt_struct_traverse(Ctx *ctx, knh_Stmt_struct *b, f_traverse gc);
@@ -881,9 +892,8 @@ Stmt* new_Stmt(Ctx *ctx, knh_flag_t flag, knh_stmt_t stt);
 Stmt* new_StmtDONE(Ctx *ctx);
 void knh_Stmt_done(Ctx *ctx, Stmt *o);
 void knh_Stmt_add(Ctx *ctx, Stmt *o, Term *tm);
-void knh_Stmt_add_ERR(Ctx *ctx, Stmt *o, Token *tk);
 Stmt *knh_Stmt_tail(Stmt *o);
-void knh_Stmt_tail_append(Ctx *ctx, Stmt *o, Stmt *stmt);
+Stmt* knh_StmtNULL_tail_append(Ctx *ctx, Stmt *o, Stmt *stmt);
 void knh_Stmt__s(Ctx *ctx, Stmt *o, OutputStream *w, String *m);
 void knh_Stmt__dump(Ctx *ctx, Stmt *o, OutputStream *w, String *m);
 /* ../class/konohac/knh_Stmt_const.c */
@@ -899,6 +909,7 @@ void knh_StmtCLASS_cmpl(Ctx *ctx, Stmt *stmt, Compiler *cpr, NameSpace *ns, knh_
 	 void knh_StmtPRINT_cmpl(Ctx *ctx, Stmt *stmt, Compiler *cpr, NameSpace *ns, knh_type_t reqt, int level);
 void knh_StmtASSERT_decl(Ctx *ctx, Stmt *stmt, Compiler *cpr, NameSpace *ns, knh_type_t reqt, int level);
 /* ../class/konohac/knh_StmtDECLLET.c */
+knh_type_t knh_Compiler_typeinfer(Ctx *ctx, Compiler *cpr, knh_type_t type, char *varname);
 Stmt *new_StmtDONE_type(Ctx *ctx, knh_type_t type);
 void knh_Token_setSFPIDX(Token *tk, int idx, knh_type_t type);
 void knh_TokenNAME_typing(Ctx *ctx, Token *tk, Compiler *cpr);
@@ -914,9 +925,8 @@ void knh_StmtFOREACH_cmpl(Ctx *ctx, Stmt *stmt, Compiler *cpr, NameSpace *ns, kn
 /* ../class/konohac/knh_StmtEXPR.c */
 void knh_Stmt_setType(Stmt *stmt, knh_type_t type);
 knh_bool_t knh_Term_isTyped(Term *tm);
-knh_type_t knh_Term_get_type(Term *tm);
+knh_type_t knh_Term_get_type(Ctx *ctx, Compiler *cpr, Term *tm);
 knh_type_t knh_Term_get_cid(Term *tm);
-Term *knh_StmtCALL1_typing(Ctx *ctx, Stmt *stmt, Compiler *cpr, NameSpace *ns, knh_class_t reqc);
 knh_methodn_t knh_Token_tomethodn(Ctx *ctx, Token *o);
 Term *knh_StmtOP_typing(Ctx *ctx, Stmt *stmt, Compiler *cpr, NameSpace *ns, knh_class_t reqc);
 Term *knh_Stmt_typing(Ctx *ctx, Stmt *stmt, Compiler *cpr, NameSpace *ns, knh_class_t reqc);
@@ -997,7 +1007,7 @@ void knh_Token_setERR(Ctx *ctx, Token *tk, String *msg);
 void knh_Token_struct_init(Ctx *ctx, knh_Token_struct *b, int init, Object *cs);
 void knh_Token_struct_traverse(Ctx *ctx, knh_Token_struct *b, f_traverse gc);
 Token* new_Token(Ctx *ctx, knh_flag_t flag, knh_fileid_t fileid, knh_line_t line, knh_token_t tt);
-Token *new_Token__text(Ctx *ctx, Token *ftk, knh_bytes_t text);
+Token *new_Token__text(Ctx *ctx, Any *fln, knh_bytes_t text);
 void knh_Token_setFL(Token *o, Any *fln);
 Token *new_TokenASIS(Ctx *ctx, Any *fln);
 Token *new_TokenCID(Ctx *ctx, Any *fln, knh_class_t cid);
@@ -1010,11 +1020,12 @@ void knh_Token_tokens_empty(Ctx *ctx, Token *o);
 char *knh_Token_tochar(Token *o);
 knh_bytes_t knh_Token_tobytes(Token *o);
 void knh_Token__s(Ctx *ctx, Token *o, OutputStream *w, String *m);
+void knh_Token__k(Ctx *ctx, Token *o, OutputStream *w, String *m);
 void knh_Token__dump(Ctx *ctx, Token *o, OutputStream *w, String *m);
 /* ../class/konohac/knh_Token_const.c */
 Token* new_TokenCONST(Ctx *ctx, Any *fln, Any *data);
 void knh_Token_setCONST(Ctx *ctx, Token *o, Any *data);
-void knh_Token_toCONST(Token *o);
+Token* knh_Token_toCONST(Token *o);
 knh_type_t knh_Token_totype(Ctx *ctx, Token *o, knh_class_t defc, NameSpace *ns);
 knh_class_t knh_Token_toclass(Ctx *ctx, Token *o, knh_class_t defc, NameSpace *ns);
 int knh_Token_isLCONSTN(Token *tk);
@@ -1024,15 +1035,12 @@ void knh_Token_constSystemVariable(Ctx *ctx, Token *tk, Compiler *cpr);
 Token *knh_Token_const(Ctx *ctx, Token *o, knh_class_t reqc, NameSpace *ns);
 /* ../class/konohac/knh_tokens_t.c */
 Token* knh_tokens_curToken(knh_tokens_t *tc);
-Token* knh_tokens_prevToken(knh_tokens_t *tc);
 int knh_tokens_count(knh_tokens_t *tc, knh_token_t tt);
 void knh_tokens_nextStmt(knh_tokens_t *tc);
-void knh_tokens_stepFirstStmt(Ctx *ctx, knh_tokens_t *tc, knh_tokens_t *sub, int needsSEMICOLON);
+knh_tokens_t knh_tokens_splitSTMT(Ctx *ctx, knh_tokens_t *tc);
+knh_tokens_t knh_tokens_splitEXPR(Ctx *ctx, knh_tokens_t *tc, knh_token_t tt);
+String *knh_tokens_toString(Ctx *ctx, knh_tokens_t *tc, char *premsg, char *postmsg);
 void knh_tokens_ignore(Ctx *ctx, knh_tokens_t *tc);
-void knh_tokens_stepFirstEXPR(Ctx *ctx, knh_tokens_t *tc, knh_token_t tt, knh_tokens_t *sub);
-void knh_tokens_stepFirstEMPTYEXPR(Ctx *ctx, knh_tokens_t *tc, knh_token_t tt, knh_tokens_t *sub);
-void knh_tokens_stepFirstComma(Ctx *ctx, knh_tokens_t *tc, knh_tokens_t *sub);
-String *knh_tokens_toString(Ctx *ctx, knh_tokens_t *tc);
 /* ../konoha/deps/cygwin/filesystem__cygwin.c */
 char * knh_format_ospath(char *buf, size_t bufsiz, knh_bytes_t path);
 char * knh_format_homepath(char *buf, size_t bufsiz);
@@ -1169,10 +1177,12 @@ void knh_Object_unlock(Ctx *ctx, Object *o);
 double knh_gettime();
 /* ../konoha/runtime/konoha_typesystem.c */
 knh_bool_t knh_class_instanceof(knh_class_t scid, knh_class_t tcid);
-METHOD knh__Object_opHasstanceof(Ctx *ctx, knh_sfp_t *sfp);
+METHOD knh__Object_opInstanceof(Ctx *ctx, knh_sfp_t *sfp);
 knh_bool_t knh_Object_opTypeOf(Ctx *ctx, Object *o, knh_type_t t);
 knh_bool_t knh_class_isSynonym(Ctx *ctx, knh_class_t scid, knh_class_t tcid);
-Object * knh_Object_opAs(Ctx *ctx, Object *o, knh_class_t tcid);
+knh_class_t knh_class_parent(knh_class_t c1, knh_class_t c2);
+METHOD knh__Object_opAs(Ctx *ctx, knh_sfp_t *sfp);
+Object* knh_Object_opAs(Ctx *ctx, Object *o, knh_class_t tcid);
 Object *knh_Object_opAs__type(Ctx *ctx, Object *o, knh_type_t reqt, Object *def);
 Exception* new_Exception__type(Ctx *ctx, Object *value, knh_type_t spec_type);
 /* ../konoha/utils/utils_file.c */
