@@ -99,7 +99,7 @@ extern "C" {
 #define TT_INSTANCEOF                   ((knh_token_t)56)
 #define TT_ISA                          ((knh_token_t)57)
 #define TT_AS                           ((knh_token_t)58)
-#define TT_IN                           ((knh_token_t)59)
+#define TT_HAS                          ((knh_token_t)59)
 #define TT_IS                           ((knh_token_t)60)
 #define TT_TO                           ((knh_token_t)61)
 #define TT_EQ                           ((knh_token_t)62)
@@ -189,8 +189,8 @@ extern "C" {
 #ifndef METHODN_opAs
 #define METHODN_opAs                    METHODN_NONAME
 #endif
-#ifndef METHODN_opIn
-#define METHODN_opIn                    METHODN_NONAME
+#ifndef METHODN_opHas
+#define METHODN_opHas                   METHODN_NONAME
 #endif
 #ifndef METHODN_opIs
 #define METHODN_opIs                    METHODN_NONAME
